/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_iem_cdd_control.sas
| PRODUCT: SAS Financial Crimes Foundation
| SYSTEM:  Windows, Unix
| DESC:    FCF to ECM Case and Incident creation and synchronization
+------------------------------------------------------------------------------+
| USAGE:   %fcf_iem_cdd_control;
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_iem_cdd_control;


    /*------------------------------------------
    * Set error code to 0 and message to blank
    *------------------------------------------*/
    %let trans_rc = 0;
    %let _fcfmsg_=;

    /*==========================================================================* 
     * Step:            Get Investigations where                                * 
     *                   INVESTIGATION_IND is Y                                 * 
     * Transform:       Extract                                                 * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    FSK_CDD_SCORING_EVENT -                                 * 
     *                   &db_kc_libname..FSK_CDD_SCORING_EVENT                  * 
     * Target Table:    All Investigations -                                    * 
     *                   work.all_investigations                                * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(&db_kc_libname..FSK_CDD_SCORING_EVENT); 

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete all_investigations;
    quit;

    proc sql;
        create view work.all_investigations as
        select
             CDD_SCORING_EVENT_KEY,
             VERSION_NO,
             SEGMENT_ID,
             PRIMARY_ENTITY_KEY,
             PRIMARY_ENTITY_NUMBER,
             PRIMARY_ENTITY_LEVEL_CODE,
             CDD_RULE_ID,
             RULE_SCORE,
             SCORE_CLASSIFICATION_CODE,
             EVENT_DATE,
             AUTO_HIGH_IND,
             INVESTIGATION_IND,
             INVESTIGATION_ACTION_CODE,
             CATEGORY_CODE,
             SCORE_TYPE_CODE,
             INVESTIGATION_TYPE_CD,
             INVESTIGATION_CATEGORY_CD,
             INVESTIGATION_SUBCATEGORY_CD,
             INVESTIGATION_RK,
             INVESTIGATION_STATUS_CD
        from &SOURCEDS
          where INVESTIGATION_IND = 'Y'
        ;
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create view from Knowledge Center;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    %let SOURCEDS = work.all_investigations;

    /*==========================================================================* 
     * Step:            Get review dates                                        * 
     * Transform:       Join                                                    * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Tables:   All Investigations -                                    * 
     *                   work.all_investigations                                * 
     *                  FSK_CDD_ENTITY_FACT -                                   * 
     *                   &db_kc_libname..FSK_CDD_ENTITY_FACT                    * 
     * Target Table:    All Investigations with dates -                         * 
     *                   work.all_investigations_w_dates                        * 
     *==========================================================================*/ 

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete all_investigations_w_dates;
    quit;

    proc sql;
        create view work.all_investigations_w_dates as
        select
            all_investigations.CDD_SCORING_EVENT_KEY,
            all_investigations.VERSION_NO,
            all_investigations.SEGMENT_ID,
            all_investigations.PRIMARY_ENTITY_KEY,
            all_investigations.PRIMARY_ENTITY_NUMBER,
            all_investigations.PRIMARY_ENTITY_LEVEL_CODE,
            all_investigations.CDD_RULE_ID,
            all_investigations.RULE_SCORE,
            all_investigations.SCORE_CLASSIFICATION_CODE,
            all_investigations.EVENT_DATE,
            all_investigations.AUTO_HIGH_IND,
            all_investigations.INVESTIGATION_IND,
            all_investigations.INVESTIGATION_ACTION_CODE,
            all_investigations.CATEGORY_CODE,
            all_investigations.SCORE_TYPE_CODE,
            all_investigations.INVESTIGATION_TYPE_CD,
            all_investigations.INVESTIGATION_CATEGORY_CD,
            all_investigations.INVESTIGATION_SUBCATEGORY_CD,
            all_investigations.INVESTIGATION_RK,
            all_investigations.INVESTIGATION_STATUS_CD,
            FSK_CDD_ENTITY_FACT.NEXT_REVIEW_DATE,
            FSK_CDD_ENTITY_FACT.LAST_REVIEW_DATE
        from
            work.all_investigations as all_investigations left join 
            &db_kc_libname..FSK_CDD_ENTITY_FACT as FSK_CDD_ENTITY_FACT on
            (
                all_investigations.SEGMENT_ID = FSK_CDD_ENTITY_FACT.SEGMENT_ID
                and all_investigations.PRIMARY_ENTITY_NUMBER = FSK_CDD_ENTITY_FACT.PRIMARY_ENTITY_NUMBER
                and all_investigations.PRIMARY_ENTITY_LEVEL_CODE = FSK_CDD_ENTITY_FACT.PRIMARY_ENTITY_LEVEL_CODE
            )
        ;
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to get review dates;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Action Code splitter                                    * 
     * Transform:       Splitter                                                * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    All Investigations with dates -                         * 
     *                   work.all_investigations_w_dates                        * 
     * Target Tables:   CDD ALERT - iem.cdd_alert                               * 
     *                  CDD CASE - iem.cdd_case                                 * 
     *                  CDD ALERT2NEWCASE -                                     * 
     *                   iem.cdd_alert2newcase                                  * 
     *                  CDD ALERT2OLDCASE -                                     * 
     *                   iem.cdd_alert2oldcase                                  * 
     *                  CDD ALERT2CASE - iem.cdd_alert2case                     * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.all_investigations_w_dates); 

    proc datasets lib=iem nolist nowarn memtype = (data view);
        delete cdd_alert;
        delete cdd_case;
        delete cdd_alert2newcase;
        delete cdd_alert2case;
    quit;

    proc datasets lib=work nolist nowarn memtype = (data view);
        delete cdd_alert2oldcase;
    quit;

    data iem.cdd_alert 
         iem.cdd_case 
         iem.cdd_alert2newcase 
         work.cdd_alert2oldcase
         iem.cdd_alert2case ;
        set &SOURCEDS;

        if INVESTIGATION_ACTION_CODE = 'INCIDENT' then 
            output iem.cdd_alert;

        if INVESTIGATION_ACTION_CODE  = 'CASE' then 
            output iem.cdd_case;

        if INVESTIGATION_ACTION_CODE  = 'INC2NEWCASE' then 
            output iem.cdd_alert2newcase;

        if INVESTIGATION_ACTION_CODE = 'INC2OLDCASE' then 
            output work.cdd_alert2oldcase;

        if INVESTIGATION_ACTION_CODE = 'INC2CASE' then 
            output iem.cdd_alert2case;
    run;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

    proc sql;
        create table work.temp_map as
        select
             CDD_SCORING_EVENT_KEY,
             VERSION_NO,
             SEGMENT_ID,
             PRIMARY_ENTITY_KEY,
             PRIMARY_ENTITY_NUMBER,
             PRIMARY_ENTITY_LEVEL_CODE,
             CDD_RULE_ID,
             RULE_SCORE,
             SCORE_CLASSIFICATION_CODE,
             EVENT_DATE,
             AUTO_HIGH_IND,
             INVESTIGATION_IND,
             INVESTIGATION_ACTION_CODE,
             CATEGORY_CODE,
             SCORE_TYPE_CODE,
             INVESTIGATION_TYPE_CD,
             INVESTIGATION_CATEGORY_CD,
             INVESTIGATION_SUBCATEGORY_CD,
             INVESTIGATION_RK,
             INVESTIGATION_STATUS_CD,
             . as SEC_INVESTIGATION_RK,
             NEXT_REVIEW_DATE,
             LAST_REVIEW_DATE
        from work.cdd_alert2oldcase
        ;
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create temporary work table;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    %let SOURCEDS = work.temp_map;

    data iem.cdd_alert2oldcase;
        set work.temp_map;
    run;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

	%fcf_get_ecm_sequence_ids(in=iem.cdd_alert2oldcase,out=iem.cdd_alert2oldcase,type=incident,block=&iem_performance_load,col=sec_investigation_rk);

    %fcf_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to generate ECM INCIDENT primary keys for ALERT2OLDCASE load;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            ALERT                                                   * 
     * Transform:       Splitter                                                * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    CDD ALERT - iem.cdd_alert                               * 
     * Target Tables:   CDD ALERT New Incidents -                               * 
     *                   work.alert_new_inc                                     * 
     *                  CDD ALERT Existing Incidents -                          * 
     *                   work.alert_exist_inc                                   * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(iem.cdd_alert); 

    proc datasets lib=work nolist nowarn memtype = (data view);
        delete alert_new_inc;
        delete alert_exist_inc;
    quit;

    data work.alert_new_inc 
         work.alert_exist_inc ;
        set &SOURCEDS;

        if INVESTIGATION_RK = . then 
            output work.alert_new_inc;

        if INVESTIGATION_RK ^= . then 
            output work.alert_exist_inc;
    run;

	%fcf_get_ecm_sequence_ids(in=work.alert_new_inc,out=work.alert_new_inc,type=incident,block=&iem_performance_load,col=investigation_rk);

    %fcf_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to generate ECM INCIDENT primary keys for ALERT load;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            CASE                                                    * 
     * Transform:       Splitter                                                * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    CDD CASE - iem.cdd_case                                 * 
     * Target Tables:   CDD CASE New Cases -                                    * 
     *                   work.case_new_case                                     * 
     *                  CDD CASE Existing Cases -                               * 
     *                   work.case_exist_case                                   * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(iem.cdd_case); 

    proc datasets lib=iem nolist nowarn memtype = (data view);
        delete case_new_case;
        delete case_exist_case;
    quit;

    data work.case_new_case 
         work.case_exist_case ;
       set &SOURCEDS;

       if INVESTIGATION_RK = . then 
          output work.case_new_case;

       if INVESTIGATION_RK ^= . then 
          output work.case_exist_case;

    run;

	%fcf_get_ecm_sequence_ids(in=work.case_new_case,out=work.case_new_case,type=case,block=&iem_performance_load,col=investigation_rk);
  
    %fcf_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to generate ECM CASE primary keys for CASE load;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            ALERT2NEWCASE                                           * 
     * Transform:       Splitter                                                * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    CDD ALERT2NEWCASE -                                     * 
     *                   iem.cdd_alert2newcase                                  * 
     * Target Tables:   CDD ALERT2NEWCASE New Cases -                           * 
     *                   work.alert2newcase_new_case                            * 
     *                  CDD ALERT2NEWCASE New Incidents -                       * 
     *                   work.alert2newcase_new_inc                             * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(iem.cdd_alert2newcase); 

    proc datasets lib=work nolist nowarn memtype = (data view);
        delete alert2newcase_new_case;
        delete alert2newcase_new_inc;
    quit;

    data work.alert2newcase_new_case 
         work.alert2newcase_new_inc;
        set &SOURCEDS;
        output work.alert2newcase_new_case;
        output work.alert2newcase_new_inc;
    run;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

    proc sql;
        create table work.temp_map as
        select
             CDD_SCORING_EVENT_KEY,
             VERSION_NO,
             SEGMENT_ID,
             PRIMARY_ENTITY_KEY,
             PRIMARY_ENTITY_NUMBER,
             PRIMARY_ENTITY_LEVEL_CODE,
             CDD_RULE_ID,
             RULE_SCORE,
             SCORE_CLASSIFICATION_CODE,
             EVENT_DATE,
             AUTO_HIGH_IND,
             INVESTIGATION_IND,
             INVESTIGATION_ACTION_CODE,
             CATEGORY_CODE,
             SCORE_TYPE_CODE,
             INVESTIGATION_TYPE_CD,
             INVESTIGATION_CATEGORY_CD,
             INVESTIGATION_SUBCATEGORY_CD,
             INVESTIGATION_RK,
             INVESTIGATION_STATUS_CD,
             . as SEC_INVESTIGATION_RK,
             NEXT_REVIEW_DATE,
             LAST_REVIEW_DATE
        from work.alert2newcase_new_inc
        ;
    quit;

    %let SOURCEDS = work.temp_map;

    data work.alert2newcase_new_inc;
        set work.temp_map;
    run;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

	%fcf_get_ecm_sequence_ids(in=work.alert2newcase_new_case,out=work.alert2newcase_new_case,type=case,block=&iem_performance_load,col=investigation_rk);

    %fcf_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to generate ECM CASE primary keys for ALERT2NEWCASE load;;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

	%fcf_get_ecm_sequence_ids(in=work.alert2newcase_new_inc,out=work.alert2newcase_new_inc,type=incident,block=&iem_performance_load,col=sec_investigation_rk);
    
    %fcf_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to generate ECM INCIDENT primary keys for ALERT2NEWCASE load;;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Inc w new CASE_RK                                       * 
     * Transform:       Join                                                    * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Tables:   CDD ALERT2NEWCASE New Cases -                           * 
     *                   work.alert2newcase_new_case                            * 
     *                  CDD ALERT2NEWCASE New Incidents -                       * 
     *                   work.alert2newcase_new_inc                             * 
     * Target Table:    CDD ALERT2NEWCASE New Incidents with                    * 
     *                  CASE_RK -                                               * 
     *                   work.alert2newcase_new_inc_w_caserk                    * 
     *==========================================================================*/ 

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete alert2newcase_new_inc_w_caserk;
    quit;

    proc sql;
        create table work.alert2newcase_new_inc_w_caserk as
        select
            alert2newcase_new_inc.CDD_SCORING_EVENT_KEY,
            alert2newcase_new_inc.VERSION_NO,
            alert2newcase_new_inc.SEGMENT_ID,
            alert2newcase_new_inc.PRIMARY_ENTITY_KEY,
            alert2newcase_new_inc.PRIMARY_ENTITY_NUMBER,
            alert2newcase_new_inc.PRIMARY_ENTITY_LEVEL_CODE,
            alert2newcase_new_inc.CDD_RULE_ID,
            alert2newcase_new_inc.RULE_SCORE,
            alert2newcase_new_inc.SCORE_CLASSIFICATION_CODE,
            alert2newcase_new_inc.EVENT_DATE,
            alert2newcase_new_inc.AUTO_HIGH_IND,
            alert2newcase_new_inc.INVESTIGATION_IND,
            alert2newcase_new_inc.INVESTIGATION_ACTION_CODE,
            alert2newcase_new_inc.CATEGORY_CODE,
            alert2newcase_new_inc.SCORE_TYPE_CODE,
            alert2newcase_new_inc.INVESTIGATION_TYPE_CD,
            alert2newcase_new_inc.INVESTIGATION_CATEGORY_CD,
            alert2newcase_new_inc.INVESTIGATION_SUBCATEGORY_CD,
            alert2newcase_new_case.INVESTIGATION_RK,
            alert2newcase_new_inc.INVESTIGATION_STATUS_CD,
            alert2newcase_new_inc.SEC_INVESTIGATION_RK,
            alert2newcase_new_inc.NEXT_REVIEW_DATE,
            alert2newcase_new_inc.LAST_REVIEW_DATE
        from
            work.alert2newcase_new_inc left join 
            work.alert2newcase_new_case on
            (
                alert2newcase_new_case.CDD_SCORING_EVENT_KEY = alert2newcase_new_inc.CDD_SCORING_EVENT_KEY
            )
        ;
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to join work.alert2newcase_new_inc and work.alert2newcase_new_case;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            ALERT2CASE                                              * 
     * Transform:       Splitter                                                * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    CDD ALERT2CASE - iem.cdd_alert2case                     * 
     * Target Tables:   CDD ALERT2CASE Existing Cases -                         * 
     *                   work.alert2case_exist_case                             * 
     *                  CDD ALERT2CASE New Cases -                              * 
     *                   work.alert2case_new_case                               * 
     *                  CDD ALERT2CASE New Alerts -                             * 
     *                   work.alert2case_new_inc                                * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(iem.cdd_alert2case); 

    proc datasets lib=work nolist nowarn memtype = (data view);
        delete alert2case_exist_case;
        delete alert2case_new_case;
        delete alert2case_new_inc;
    quit;

    data work.alert2case_exist_case 
         work.alert2case_new_case 
         work.alert2case_new_inc;
       set &SOURCEDS;

       if INVESTIGATION_RK ^= . then 
          output work.alert2case_exist_case;
       if INVESTIGATION_RK = . then 
          output work.alert2case_new_case;

       output work.alert2case_new_inc;
    run;


    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

    proc sql;
        create table work.temp_map as
        select
             CDD_SCORING_EVENT_KEY,
             VERSION_NO,
             SEGMENT_ID,
             PRIMARY_ENTITY_KEY,
             PRIMARY_ENTITY_NUMBER,
             PRIMARY_ENTITY_LEVEL_CODE,
             CDD_RULE_ID,
             RULE_SCORE,
             SCORE_CLASSIFICATION_CODE,
             EVENT_DATE,
             AUTO_HIGH_IND,
             INVESTIGATION_IND,
             INVESTIGATION_ACTION_CODE,
             CATEGORY_CODE,
             SCORE_TYPE_CODE,
             INVESTIGATION_TYPE_CD,
             INVESTIGATION_CATEGORY_CD,
             INVESTIGATION_SUBCATEGORY_CD,
             INVESTIGATION_RK,
             INVESTIGATION_STATUS_CD,
             . as SEC_INVESTIGATION_RK,
             NEXT_REVIEW_DATE,
             LAST_REVIEW_DATE
        from work.alert2case_new_inc
        ;
    quit;

    %let SOURCEDS = work.temp_map;

    data work.alert2case_new_inc;
        set work.temp_map;
    run;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

	%fcf_get_ecm_sequence_ids(in=work.alert2case_new_case,out=work.alert2case_new_case,type=case,block=&iem_performance_load,col=investigation_rk);

    %fcf_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to generate ECM CASE primary keys for ALERT2CASE load;;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

	%fcf_get_ecm_sequence_ids(in=work.alert2case_new_inc,out=work.alert2case_new_inc,type=incident,block=&iem_performance_load,col=sec_investigation_rk);
  
    %fcf_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to generate ECM INCIDENT primary keys for ALERT2CASE load;;
        %fcf_put_error;
        %goto MACRO_END;
    %end;
	
    /*==========================================================================* 
     * Step:            Inc w existing CASE_RK                                  * 
     * Transform:       Join                                                    * 
     *                                                                          * 
     * Source Tables:   CDD ALERT2CASE New Alerts -                             * 
     *                   work.alert2case_new_inc                                * 
     *                  CDD ALERT2CASE Existing Cases -                         * 
     *                   work.alert2case_exist_case                             * 
     * Target Table:    CDD ALERT2CASE New Incidents with                       * 
     *                  Existing CASE_RK -                                      * 
     *                  work.alert2case_new_inc_w_exst_caserk                   * 
     *==========================================================================*/ 

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete alert2case_new_inc_w_exst_caserk;
    quit;

    proc sql;
        create view work.alert2case_new_inc_w_exst_caserk as
        select
            alert2case_new_inc.CDD_SCORING_EVENT_KEY,
            alert2case_new_inc.VERSION_NO,
            alert2case_new_inc.SEGMENT_ID,
            alert2case_new_inc.PRIMARY_ENTITY_KEY,
            alert2case_new_inc.PRIMARY_ENTITY_NUMBER,
            alert2case_new_inc.PRIMARY_ENTITY_LEVEL_CODE,
            alert2case_new_inc.CDD_RULE_ID,
            alert2case_new_inc.RULE_SCORE,
            alert2case_new_inc.SCORE_CLASSIFICATION_CODE,
            alert2case_new_inc.EVENT_DATE,
            alert2case_new_inc.AUTO_HIGH_IND,
            alert2case_new_inc.INVESTIGATION_IND,
            alert2case_new_inc.INVESTIGATION_ACTION_CODE,
            alert2case_new_inc.CATEGORY_CODE,
            alert2case_new_inc.SCORE_TYPE_CODE,
            alert2case_new_inc.INVESTIGATION_TYPE_CD,
            alert2case_new_inc.INVESTIGATION_CATEGORY_CD,
            alert2case_new_inc.INVESTIGATION_SUBCATEGORY_CD,
            alert2case_new_inc.INVESTIGATION_STATUS_CD,
            alert2case_new_inc.SEC_INVESTIGATION_RK,
            alert2case_exist_case.INVESTIGATION_RK,
            alert2case_new_inc.NEXT_REVIEW_DATE,
            alert2case_new_inc.LAST_REVIEW_DATE
        from
            work.alert2case_new_inc left join 
            work.alert2case_exist_case on
            (
                alert2case_new_inc.CDD_SCORING_EVENT_KEY = alert2case_exist_case.CDD_SCORING_EVENT_KEY
            )
        ;
    quit;

    %fcf_rcset(&sqlrc);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to join work.alert2case_new_inc and work.alert2case_exist_case;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Inc w new CASE_RK                                       * 
     * Transform:       Join                                                    * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Tables:   CDD ALERT2CASE New Alerts -                             * 
     *                   work.alert2case_new_inc                                * 
     *                  CDD ALERT2CASE New Cases -                              * 
     *                   work.alert2case_new_case                               * 
     * Target Table:    CDD ALERT2CASE New Incidents with New                   * 
     *                  CASE_RK -                                               * 
     *                   work.alert2case_new_inc_w_new_caserk                   * 
     *==========================================================================*/ 

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete alert2case_new_inc_w_new_caserk;
    quit;

    proc sql;
        create view work.alert2case_new_inc_w_new_caserk as
        select
            alert2case_new_inc.CDD_SCORING_EVENT_KEY,
            alert2case_new_inc.VERSION_NO,
            alert2case_new_inc.SEGMENT_ID,
            alert2case_new_inc.PRIMARY_ENTITY_KEY,
            alert2case_new_inc.PRIMARY_ENTITY_NUMBER,
            alert2case_new_inc.PRIMARY_ENTITY_LEVEL_CODE,
            alert2case_new_inc.CDD_RULE_ID,
            alert2case_new_inc.RULE_SCORE,
            alert2case_new_inc.SCORE_CLASSIFICATION_CODE,
            alert2case_new_inc.EVENT_DATE,
            alert2case_new_inc.AUTO_HIGH_IND,
            alert2case_new_inc.INVESTIGATION_IND,
            alert2case_new_inc.INVESTIGATION_ACTION_CODE,
            alert2case_new_inc.CATEGORY_CODE,
            alert2case_new_inc.SCORE_TYPE_CODE,
            alert2case_new_inc.INVESTIGATION_TYPE_CD,
            alert2case_new_inc.INVESTIGATION_CATEGORY_CD,
            alert2case_new_inc.INVESTIGATION_SUBCATEGORY_CD,
            alert2case_new_inc.INVESTIGATION_STATUS_CD,
            alert2case_new_inc.SEC_INVESTIGATION_RK,
            alert2case_new_case.INVESTIGATION_RK,
            alert2case_new_inc.NEXT_REVIEW_DATE,
            alert2case_new_inc.LAST_REVIEW_DATE
        from
            work.alert2case_new_inc left join 
            work.alert2case_new_case on
            (
                alert2case_new_inc.CDD_SCORING_EVENT_KEY = alert2case_new_case.CDD_SCORING_EVENT_KEY
            )
        ;
    quit;

    %fcf_rcset(&sqlrc);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to join work.alert2case_new_inc and work.alert2case_new_case;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Inc w all CASE_RK                                       * 
     * Transform:       Append                                                  * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Tables:   CDD ALERT2CASE New Incidents with                       * 
     *                  Existing CASE_RK -                                      * 
     *                  work.alert2case_new_inc_w_exst_case                     * 
     *                  rk                                                      * 
     *                  CDD ALERT2CASE New Incidents with New                   * 
     *                  CASE_RK -                                               * 
     *                   work.alert2case_new_inc_w_new_caserk                   * 
     * Target Table:    CDD ALERT2CASE Incidents with All                       * 
     *                  CASE_RK -                                               * 
     *                   work.alert2case_new_inc_w_caserk                       * 
     *==========================================================================*/ 

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete alert2case_new_inc_w_caserk;
    quit;

    data work.alert2case_new_inc_w_caserk;
        set work.alert2case_new_inc_w_exst_caserk
            work.alert2case_new_inc_w_new_caserk;
        keep CDD_SCORING_EVENT_KEY VERSION_NO SEGMENT_ID PRIMARY_ENTITY_KEY 
            PRIMARY_ENTITY_NUMBER PRIMARY_ENTITY_LEVEL_CODE CDD_RULE_ID RULE_SCORE 
            SCORE_CLASSIFICATION_CODE EVENT_DATE AUTO_HIGH_IND INVESTIGATION_IND 
            INVESTIGATION_ACTION_CODE CATEGORY_CODE SCORE_TYPE_CODE INVESTIGATION_TYPE_CD 
            INVESTIGATION_CATEGORY_CD INVESTIGATION_SUBCATEGORY_CD 
            INVESTIGATION_STATUS_CD SEC_INVESTIGATION_RK INVESTIGATION_RK 
            NEXT_REVIEW_DATE LAST_REVIEW_DATE; 
    run;

    /*==========================================================================* 
     * Step:            Load Incident                                           * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    CDD ALERT New Incidents -                               * 
     *                   work.alert_new_inc                                     * 
     * Target Table:    New CDD Incidents -                                     * 
     *                   iem.cdd_new_incidents                                  * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.alert_new_inc); 

    %if %fcf_get_observation_count(table=work.alert_new_inc,SASDATASET=Y) lt 1 %then %goto skipped1;

	data _null_;
     	call symput('current_dt',put(datetime(),datetime22.));
	run;

    proc datasets lib = iem nolist nowarn memtype = (data view);
        delete cdd_new_incidents;
    quit;

	data iem.cdd_new_incidents;
		if 0 then set ecm_db.incident_live;
		set &SOURCEDS. (keep= CDD_SCORING_EVENT_KEY SEGMENT_ID PRIMARY_ENTITY_NUMBER EVENT_DATE INVESTIGATION_TYPE_CD INVESTIGATION_CATEGORY_CD INVESTIGATION_SUBCATEGORY_CD INVESTIGATION_RK);
		INCIDENT_RK = INVESTIGATION_RK;
		VALID_FROM_DTTM = "&current_dt"dt;
		INCIDENT_ID = CATX('-','CDD',compress(put(CDD_SCORING_EVENT_KEY,32.)));
		SOURCE_SYSTEM_CD = SEGMENT_ID;
		INCIDENT_TYPE_CD = INVESTIGATION_TYPE_CD;
		INCIDENT_CATEGORY_CD = INVESTIGATION_CATEGORY_CD;
		INCIDENT_SUBCATEGORY_CD = INVESTIGATION_SUBCATEGORY_CD;
		CREATE_USER_ID = "&ecm_user";
		CREATE_DTTM =  EVENT_DATE;
		VERSION_NO = 1;
		DELETE_FLG = "0";
		PRIMARY_ENTITY_NUMBER = PRIMARY_ENTITY_NUMBER;
		drop CDD_SCORING_EVENT_KEY SEGMENT_ID EVENT_DATE INVESTIGATION_TYPE_CD INVESTIGATION_CATEGORY_CD INVESTIGATION_SUBCATEGORY_CD INVESTIGATION_RK;
	run;

    %fcf_rcset(&syserr);
    
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create IEM.NEW_INCIDENTS for ALT load;
        %fcf_put_error;
        %goto MACRO_END;
    %end;
      
    /*%fcf_iem_cdd_load_inc(ds=iem.cdd_new_incidents);*/
    /*%fcf_iem_cdd_upd_sc_event(ds=work.alert_new_inc,tracking=alert);*/

    %skipped1:


    /*==========================================================================* 
     * Step:            Update Incident                                         * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    CDD ALERT Existing Incidents -                          * 
     *                   work.alert_exist_inc                                   * 
     * Target Table:    Update CDD Incidents -                                  * 
     *                   iem.cdd_upd_incidents                                  * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.alert_exist_inc); 

    %if %fcf_get_observation_count(table=work.alert_exist_inc,SASDATASET=Y) lt 1 %then %goto skipped2;

	data _null_;
     	call symput('current_dt',put(datetime(),datetime22.));
	run;

    proc datasets lib = iem nolist nowarn memtype = (data view);
        delete new_incidents;
    quit;

	data iem.cdd_upd_incidents;
		if 0 then set ecm_db.incident_live;
		set &SOURCEDS. (keep=INVESTIGATION_RK EVENT_DATE);
		INCIDENT_RK = INVESTIGATION_RK;
		CREATE_DTTM =  EVENT_DATE;
		drop INVESTIGATION_RK EVENT_DATE;
	run;

    %fcf_rcset(&syserr);
    
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create IEM.NEW_INCIDENTS for ALT load;
        %fcf_put_error;
        %goto MACRO_END;
    %end;	

    /*%fcf_iem_cdd_update_inc(ds=iem.cdd_upd_incidents);*/

    %skipped2:

    /*==========================================================================* 
     * Step:            Load Case                                               * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    CDD CASE New Cases -                                    * 
     *                   work.case_new_case                                     * 
     * Target Table:    New CDD Cases - iem.cdd_new_cases                       * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.case_new_case); 

    %if %fcf_get_observation_count(table=work.case_new_case,SASDATASET=Y) lt 1 %then %goto skipped3;

	data _null_;
     	call symput('current_dt',put(datetime(),datetime22.));
	run;
 
	proc datasets lib = iem nolist nowarn memtype = (data view);
        delete cdd_new_cases;
    quit;

	data iem.cdd_new_cases;
		if 0 then set ecm_db.case_live;
		set &SOURCEDS. (keep= CDD_SCORING_EVENT_KEY SEGMENT_ID PRIMARY_ENTITY_NUMBER EVENT_DATE INVESTIGATION_TYPE_CD INVESTIGATION_CATEGORY_CD INVESTIGATION_SUBCATEGORY_CD INVESTIGATION_RK SCORE_CLASSIFICATION_CODE NEXT_REVIEW_DATE LAST_REVIEW_DATE);
		CASE_RK = INVESTIGATION_RK;
		VALID_FROM_DTTM = "&current_dt"dt;
		CASE_ID = CATX('-','CDD',compress(put(CDD_SCORING_EVENT_KEY,32.)));
		SOURCE_SYSTEM_CD = SEGMENT_ID;
		CASE_TYPE_CD = INVESTIGATION_TYPE_CD;
		CASE_CATEGORY_CD = INVESTIGATION_CATEGORY_CD;
		CASE_SUBCATEGORY_CD = INVESTIGATION_SUBCATEGORY_CD;
		PRIORITY_CD = SCORE_CLASSIFICATION_CODE;
		REGULATORY_RPT_RQD_FLG = "0";
		CREATE_USER_ID = "&ecm_user";
		CREATE_DTTM =  EVENT_DATE;
		VERSION_NO = 1;
		DELETE_FLG = "0";
		PRIMARY_ENTITY_NUMBER = PRIMARY_ENTITY_NUMBER;
		NEXT_REVIEW_DATE = NEXT_REVIEW_DATE;
		LAST_REVIEW_DATE = LAST_REVIEW_DATE;
		drop CDD_SCORING_EVENT_KEY SEGMENT_ID EVENT_DATE INVESTIGATION_TYPE_CD INVESTIGATION_CATEGORY_CD INVESTIGATION_SUBCATEGORY_CD INVESTIGATION_RK SCORE_CLASSIFICATION_CODE;
	run;

    %fcf_rcset(&syserr);
    
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create IEM.NEW_INCIDENTS for CDD load;
        %fcf_put_error;
        %goto MACRO_END;
    %end;
	

    %fcf_iem_cdd_load_case(ds=iem.cdd_new_cases);
    %fcf_iem_cdd_upd_sc_event(ds=work.case_new_case,tracking=case);

    %skipped3:

    /*==========================================================================* 
     * Step:            Update Case                                             * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    CDD CASE Existing Cases -                               * 
     *                   work.case_exist_case                                   * 
     * Target Table:    Update CDD Cases - iem.cdd_upd_cases                    * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.case_exist_case); 

    %if %fcf_get_observation_count(table=work.case_exist_case,SASDATASET=Y) lt 1 %then %goto skipped4;

	data _null_;
     	call symput('current_dt',put(datetime(),datetime22.));
	run;

    proc datasets lib = iem nolist nowarn memtype = (data view);
        delete cdd_upd_case;
    quit;

	data iem.cdd_upd_case;
		if 0 then set ecm_db.case_live;
		set &SOURCEDS. (keep= INVESTIGATION_RK NEXT_REVIEW_DATE LAST_REVIEW_DATE);
		CASE_RK = INVESTIGATION_RK;
		NEXT_REVIEW_DATE = NEXT_REVIEW_DATE;
		LAST_REVIEW_DATE = LAST_REVIEW_DATE;
		drop INVESTIGATION_RK;
	run;

    %fcf_rcset(&syserr);
    
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create IEM.NEW_INCIDENTS for CDD load;
        %fcf_put_error;
        %goto MACRO_END;
    %end;


    /*%fcf_iem_cdd_update_case(ds=iem.cdd_upd_cases);*/
 
    %skipped4:

    /*==========================================================================* 
     * Step:            Load Incident                                           * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    CDD ALERT2OLDCASE -                                     * 
     *                   iem.cdd_alert2oldcase                                  * 
     * Target Table:    New CDD Incidents -                                     * 
     *                   iem.cdd_new_incidents                                  * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(iem.cdd_alert2oldcase); 

    %if %fcf_get_observation_count(table=iem.cdd_alert2oldcase,SASDATASET=Y) lt 1 %then %goto skipped5;

	data _null_;
     	call symput('current_dt',put(datetime(),datetime22.));
	run;

    proc datasets lib = iem nolist nowarn memtype = (data view);
        delete cdd_new_incidents;
    quit;

	data iem.cdd_new_incidents;
		if 0 then set ecm_db.incident_live;
		set &SOURCEDS. (keep= CDD_SCORING_EVENT_KEY SEGMENT_ID PRIMARY_ENTITY_NUMBER EVENT_DATE INVESTIGATION_TYPE_CD INVESTIGATION_CATEGORY_CD INVESTIGATION_SUBCATEGORY_CD INVESTIGATION_RK SEC_INVESTIGATION_RK);
		INCIDENT_RK = SEC_INVESTIGATION_RK;
		VALID_FROM_DTTM = "&current_dt"dt;
		CASE_RK = INVESTIGATION_RK
		INCIDENT_ID = CATX('-','CDD',compress(put(CDD_SCORING_EVENT_KEY,32.)));
		SOURCE_SYSTEM_CD = SEGMENT_ID;
		INCIDENT_TYPE_CD = INVESTIGATION_TYPE_CD;
		INCIDENT_CATEGORY_CD = INVESTIGATION_CATEGORY_CD;
		INCIDENT_SUBCATEGORY_CD = INVESTIGATION_SUBCATEGORY_CD;
		CREATE_USER_ID = "&ecm_user";
		CREATE_DTTM =  EVENT_DATE;
		VERSION_NO = 1;
		DELETE_FLG = "0";
		PRIMARY_ENTITY_NUMBER = PRIMARY_ENTITY_NUMBER;
		drop CDD_SCORING_EVENT_KEY SEGMENT_ID EVENT_DATE INVESTIGATION_TYPE_CD INVESTIGATION_CATEGORY_CD INVESTIGATION_SUBCATEGORY_CD INVESTIGATION_RK SEC_INVESTIGATION_RK;
	run;

    %fcf_rcset(&syserr);
    
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create IEM.NEW_INCIDENTS for CDD load;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*%fcf_iem_cdd_load_inc(ds=iem.cdd_new_incidents);*/
    /*%fcf_iem_cdd_upd_sc_event(ds=iem.cdd_alert2oldcase,tracking=case);*/

    %skipped5:

    /*==========================================================================* 
     * Step:            Load Case                                               * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    CDD ALERT2NEWCASE New Cases -                           * 
     *                   work.alert2newcase_new_case                            * 
     * Target Table:    New CDD Cases - iem.cdd_new_cases                       * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.alert2newcase_new_case); 

    %if %fcf_get_observation_count(table=work.alert2newcase_new_case,SASDATASET=Y) lt 1 %then %goto skipped6;

	data _null_;
     	call symput('current_dt',put(datetime(),datetime22.));
	run;
 
	proc datasets lib = iem nolist nowarn memtype = (data view);
        delete cdd_new_cases;
    quit;

	data iem.cdd_new_cases;
		if 0 then set ecm_db.case_live;
		set &SOURCEDS. (keep= CDD_SCORING_EVENT_KEY SEGMENT_ID PRIMARY_ENTITY_NUMBER EVENT_DATE CASE_TYPE_CD CASE_CATEGORY_CD CASE_SUBCATEGORY_CD INVESTIGATION_RK SCORE_CLASSIFICATION_CODE);
		CASE_RK = INVESTIGATION_RK;
		VALID_FROM_DTTM = "&current_dt"dt;
		CASE_ID = CATX('-','CDD',compress(put(CDD_SCORING_EVENT_KEY,32.)));
		SOURCE_SYSTEM_CD = SEGMENT_ID;
		CASE_TYPE_CD = INVESTIGATION_TYPE_CD;
		CASE_CATEGORY_CD = INVESTIGATION_CATEGORY_CD;
		CASE_SUBCATEGORY_CD = INVESTIGATION_SUBCATEGORY_CD;
		PRIORITY_CD = SCORE_CLASSIFICATION_CODE;
		REGULATORY_RPT_RQD_FLG = "0";
		CREATE_USER_ID = "&ecm_user";
		CREATE_DTTM =  EVENT_DATE;
		VERSION_NO = 1;
		DELETE_FLG = "0";
		PRIMARY_ENTITY_NUMBER = PRIMARY_ENTITY_NUMBER;
		drop CDD_SCORING_EVENT_KEY SEGMENT_ID PRIMARY_ENTITY_NUMBER EVENT_DATE CASE_TYPE_CD CASE_CATEGORY_CD CASE_SUBCATEGORY_CD INVESTIGATION_RK SCORE_CLASSIFICATION_CODE;
	run;

    %fcf_rcset(&syserr);
    
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create IEM.NEW_INCIDENTS for CDD load;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    %fcf_iem_cdd_load_case(ds=iem.cdd_new_cases);
    %fcf_iem_cdd_upd_sc_event(ds=work.alert2newcase_new_case,tracking=case);

    %skipped6:

    /*==========================================================================* 
     * Step:            Load Incident                                           * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    CDD ALERT2NEWCASE New Incidents with                    * 
     *                  CASE_RK -                                               * 
     *                   work.alert2newcase_new_inc_w_caserk                    * 
     * Target Table:    New CDD Incidents -                                     * 
     *                   iem.cdd_new_incidents                                  * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.alert2newcase_new_inc_w_caserk); 

    %if %fcf_get_observation_count(table=work.alert2newcase_new_inc_w_caserk,SASDATASET=Y) lt 1 %then %goto skipped7;

 	data _null_;
     	call symput('current_dt',put(datetime(),datetime22.));
	run;

    proc datasets lib = iem nolist nowarn memtype = (data view);
        delete cdd_new_incidents;
    quit;

	data iem.cdd_new_incidents;
		if 0 then set ecm_db.incident_live;
		set &SOURCEDS. (keep= CDD_SCORING_EVENT_KEY SEGMENT_ID PRIMARY_ENTITY_NUMBER EVENT_DATE INVESTIGATION_TYPE_CD INVESTIGATION_CATEGORY_CD INVESTIGATION_SUBCATEGORY_CD INVESTIGATION_RK SEC_INVESTIGATION_RK CASE_RK);
		INCIDENT_RK = SEC_INVESTIGATION_RK;
		VALID_FROM_DTTM = "&current_dt"dt;
		CASE_RK = INVESTIGATION_RK
		INCIDENT_ID = CATX('-','CDD',compress(put(CDD_SCORING_EVENT_KEY,32.)));
		SOURCE_SYSTEM_CD = SEGMENT_ID;
		INCIDENT_TYPE_CD = INVESTIGATION_TYPE_CD;
		INCIDENT_CATEGORY_CD = INVESTIGATION_CATEGORY_CD;
		INCIDENT_SUBCATEGORY_CD = INVESTIGATION_SUBCATEGORY_CD;
		CREATE_USER_ID = "&ecm_user";
		CREATE_DTTM = EVENT_DATE;
		VERSION_NO = 1;
		DELETE_FLG = "0";
		PRIMARY_ENTITY_NUMBER = PRIMARY_ENTITY_NUMBER;
		drop CDD_SCORING_EVENT_KEY SEGMENT_ID EVENT_DATE INVESTIGATION_TYPE_CD INVESTIGATION_CATEGORY_CD INVESTIGATION_SUBCATEGORY_CD INVESTIGATION_RK SEC_INVESTIGATION_RK CASE_RK;
	run;

    %fcf_rcset(&syserr);
    
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create IEM.NEW_INCIDENTS for CDD load;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*%fcf_iem_cdd_load_inc(ds=iem.cdd_new_incidents);*/

    %skipped7:

    /*==========================================================================* 
     * Step:            Update Case                                             * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    CDD ALERT2CASE Existing Cases -                         * 
     *                   work.alert2case_exist_case                             * 
     * Target Table:    Update CDD Cases - iem.cdd_upd_cases                    * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.alert2case_exist_case); 

    %if %fcf_get_observation_count(table=work.alert2case_exist_case ,SASDATASET=Y) lt 1 %then %goto skipped8;

	data _null_;
     	call symput('current_dt',put(datetime(),datetime22.));
	run;

    proc datasets lib = iem nolist nowarn memtype = (data view);
        delete cdd_upd_case;
    quit;

	data iem.cdd_upd_case;
		if 0 then set ecm_db.case_live;
		set &SOURCEDS. (keep= INVESTIGATION_RK NEXT_REVIEW_DATE LAST_REVIEW_DATE);
		CASE_RK = INVESTIGATION_RK;
		NEXT_REVIEW_DATE = NEXT_REVIEW_DATE;
		LAST_REVIEW_DATE = LAST_REVIEW_DATE;
		drop INVESTIGATION_RK;
	run;

    %fcf_rcset(&syserr);
    
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create IEM.NEW_INCIDENTS for CDD load;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*%fcf_iem_cdd_upd_case(ds=iem.cdd_upd_cases);*/

    %skipped8:

    /*==========================================================================* 
     * Step:            Load Case                                               * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    CDD ALERT2CASE New Cases -                              * 
     *                   work.alert2case_new_case                               * 
     * Target Table:    New CDD Cases - iem.cdd_new_cases                       * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.alert2case_new_case); 

    %if %fcf_get_observation_count(table=work.alert2case_new_case,SASDATASET=Y) lt 1 %then %goto skipped9;

	data _null_;
     	call symput('current_dt',put(datetime(),datetime22.));
	run;
 
	proc datasets lib = iem nolist nowarn memtype = (data view);
        delete cdd_new_cases;
    quit;

	data iem.cdd_new_cases;
		if 0 then set ecm_db.case_live;
		set &SOURCEDS. (keep= CDD_SCORING_EVENT_KEY SEGMENT_ID PRIMARY_ENTITY_NUMBER EVENT_DATE CASE_TYPE_CD CASE_CATEGORY_CD CASE_SUBCATEGORY_CD INVESTIGATION_RK SCORE_CLASSIFICATION_CODE);
		CASE_RK = INVESTIGATION_RK;
		VALID_FROM_DTTM = "&current_dt"dt;
		CASE_ID = CATX('-','CDD',compress(put(CDD_SCORING_EVENT_KEY,32.)));
		SOURCE_SYSTEM_CD = SEGMENT_ID;
		CASE_TYPE_CD = INVESTIGATION_TYPE_CD;
		CASE_CATEGORY_CD = INVESTIGATION_CATEGORY_CD;
		CASE_SUBCATEGORY_CD = INVESTIGATION_SUBCATEGORY_CD;
		PRIORITY_CD = SCORE_CLASSIFICATION_CODE;
		REGULATORY_RPT_RQD_FLG = "0";
		CREATE_USER_ID = "&ecm_user";
		CREATE_DTTM =  EVENT_DATE;
		VERSION_NO = 1;
		DELETE_FLG = "0";
		PRIMARY_ENTITY_NUMBER = PRIMARY_ENTITY_NUMBER;
		drop CDD_SCORING_EVENT_KEY SEGMENT_ID PRIMARY_ENTITY_NUMBER EVENT_DATE CASE_TYPE_CD CASE_CATEGORY_CD CASE_SUBCATEGORY_CD INVESTIGATION_RK SCORE_CLASSIFICATION_CODE;
	run;

    %fcf_rcset(&syserr);
    
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create IEM.NEW_INCIDENTS for CDD load;
        %fcf_put_error;
        %goto MACRO_END;
    %end;
	
    %fcf_iem_cdd_load_case(ds=iem.cdd_new_cases);
    %fcf_iem_cdd_upd_sc_event(ds=work.alert2case_new_case,tracking=case);

    %skipped9:

    /*==========================================================================* 
     * Step:            Load Incident                                           * 
     * Transform:       Table Loader (Version 2.1)                              *
     *                                                                          * 
     * Source Table:    CDD ALERT2CASE Incidents with All                       * 
     *                  CASE_RK -                                               * 
     *                   work.alert2case_new_inc_w_caserk                       * 
     * Target Table:    New CDD Incidents -                                     * 
     *                   iem.cdd_new_incidents                                  *
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.alert2case_new_inc_w_caserk); 

    %if %fcf_get_observation_count(table=work.alert2case_new_inc_w_caserk,SASDATASET=Y) lt 1 %then %goto skipped10;

	data _null_;
     	call symput('current_dt',put(datetime(),datetime22.));
	run;

    proc datasets lib = iem nolist nowarn memtype = (data view);
        delete cdd_new_incidents;
    quit;

	data iem.cdd_new_incidents;
		if 0 then set ecm_db.incident_live;
		set &SOURCEDS. (keep= CDD_SCORING_EVENT_KEY SEGMENT_ID PRIMARY_ENTITY_NUMBER EVENT_DATE INVESTIGATION_TYPE_CD INVESTIGATION_CATEGORY_CD INVESTIGATION_SUBCATEGORY_CD INVESTIGATION_RK SEC_INVESTIGATION_RK);
		INCIDENT_RK = SEC_INVESTIGATION_RK;
		VALID_FROM_DTTM = "&current_dt"dt;
		CASE_RK = INVESTIGATION_RK
		INCIDENT_ID = CATX('-','CDD',compress(put(CDD_SCORING_EVENT_KEY,32.)));
		SOURCE_SYSTEM_CD = SEGMENT_ID;
		INCIDENT_TYPE_CD = INVESTIGATION_TYPE_CD;
		INCIDENT_CATEGORY_CD = INVESTIGATION_CATEGORY_CD;
		INCIDENT_SUBCATEGORY_CD = INVESTIGATION_SUBCATEGORY_CD;
		CREATE_USER_ID = "&ecm_user";
		CREATE_DTTM =  EVENT_DATE;
		VERSION_NO = 1;
		DELETE_FLG = "0";
		PRIMARY_ENTITY_NUMBER = PRIMARY_ENTITY_NUMBER;
		drop CDD_SCORING_EVENT_KEY SEGMENT_ID EVENT_DATE INVESTIGATION_TYPE_CD INVESTIGATION_CATEGORY_CD INVESTIGATION_SUBCATEGORY_CD INVESTIGATION_RK SEC_INVESTIGATION_RK;
	run;

    %fcf_rcset(&syserr);
    
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create IEM.NEW_INCIDENTS for CDD load;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*%fcf_iem_cdd_load_inc(ds=iem.cdd_new_incidents);*/

    %skipped10:

    %MACRO_END:

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
        %fcf_execute_macro(fcf_error);
    %else %do;
        %fcf_execute_macro(fcf_success);
    %end;

%mend;
