/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_iem_cdd_update_closed.sas
| PRODUCT: SAS Financial Crimes Foundation
| SYSTEM:  Windows, Unix
| DESC:    FCF to ECM Case and Incident creation and synchronization
+------------------------------------------------------------------------------+
| USAGE:   %fcf_iem_cdd_update_closed;
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_iem_cdd_update_closed;

    /*------------------------------------------
    * Set error code to 0 and message to blank
    *------------------------------------------*/
    %let trans_rc = 0;
    %let _fcfmsg_=;
    
    
    /*==========================================================================* 
     * Step:            Get All Closed Cases                                    * 
     * Transform:       Extract                                                 * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    CASE_LIVE - ecm_db.CASE_LIVE                            * 
     * Target Table:    Get All Closed Cases -                                  * 
     *                   work.all_closed                                        * 
     *==========================================================================*/ 
    
    %let SOURCEDS = %nrquote(ecm_db.CASE_LIVE); 
    
    proc datasets lib = work nolist nowarn memtype = (data view);
       delete all_closed;
    quit;
    
    proc sql;
        create table work.all_closed as
            select
                CASE_RK,
                VALID_FROM_DTTM,
                VALID_TO_DTTM,
                (input((substrn(CASE_ID,5)),best.)) as CASE_ID,
                SOURCE_SYSTEM_CD,
                CASE_TYPE_CD,
                CASE_CATEGORY_CD,
                CASE_SUBCATEGORY_CD,
                CASE_STATUS_CD,
                CASE_DISPOSITION_CD,
                OPEN_DTTM,
                REOPEN_DTTM,
                CLOSE_DTTM,
                CREATE_DTTM,
                CASE_DESC,
                CASE_LINK_SK,
                PRIORITY_CD,
                REGULATORY_RPT_RQD_FLG,
                INVESTIGATOR_USER_ID,
                UI_DEF_FILE_NM,
                CREATE_USER_ID,
                UPDATE_USER_ID,
                VERSION_NO,
                DELETE_FLG
            from &SOURCEDS
            where CASE_TYPE_CD='CDDINV' and CASE_ID like 'CDD-%' and CASE_STATUS_CD in ('C','T')
        ;
    quit;
    
    %let SOURCEDS = work.all_closed;
        
    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable retrieve all closed CDD Cases from ECM;
        %fcf_put_error;
        %goto MACRO_END;
    %end; 
    
    
    /*==========================================================================* 
     * Step:            Get All Rating Overrides                                * 
     * Transform:       Extract                                                 * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    CASE_UDF_CHAR_VALUE -                                   * 
     *                   ecm_db.CASE_UDF_CHAR_VALUE                             * 
     * Target Table:    All Rating Overrides -                                  * 
     *                   work.all_rating_overrides                              * 
     *==========================================================================*/ 
    
    %let SOURCEDS = %nrquote(ecm_db.CASE_UDF_CHAR_VALUE); 
    
    proc datasets lib = work nolist nowarn memtype = (data view);
       delete all_rating_overrides;
    quit;
    
    proc sql;
       create table work.all_rating_overrides as
          select
             CASE_RK,
             VALID_FROM_DTTM,
             UDF_VALUE length=&fcf_max_udf_len as RATING_OVERRIDE
       from &SOURCEDS
          where UDF_NM = 'RATING_OVERRIDE'
       ;
    quit;
        
    %fcf_rcset(&sqlrc); 
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to retrieve RATING_OVERRIDE UDFs from ECM;
        %fcf_put_error;
        %goto MACRO_END;
    %end;
        
    
    /*==========================================================================* 
     * Step:            Get All Party Numbers                                   * 
     * Transform:       Extract                                                 * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    CASE_UDF_CHAR_VALUE -                                   * 
     *                   ecm_db.CASE_UDF_CHAR_VALUE                             * 
     * Target Table:    All Party Numbers -                                     * 
     *                   work.all_party_nos                                     * 
     *==========================================================================*/ 
        
    %let SOURCEDS = %nrquote(ecm_db.CASE_UDF_CHAR_VALUE); 
    
    proc datasets lib = work nolist nowarn memtype = (data view);
       delete all_party_nos;
    quit;
    
    proc sql;
       create table work.all_party_nos as
          select distinct
             CASE_RK,
             VALID_FROM_DTTM,
             UDF_TABLE_NM,
             UDF_VALUE length=&fcf_max_udf_len as PRIMARY_ENTITY_NUMBER
       from &SOURCEDS
          where UDF_NM = 'PRIMARY_ENTITY_NUMBER'
       ;
    quit;
    
    %let SOURCEDS = work.all_party_nos;
        
    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to retrieve PRIMARY_ENTITY_NUMBER UDFs from ECM;
        %fcf_put_error;
        %goto MACRO_END;
    %end; 
    
    
    /*==========================================================================* 
     * Step:            Join                                                    * 
     * Transform:       Join                                                    * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Tables:   Get All Closed Cases -                                  * 
     *                   work.all_closed                                        * 
     *                  All Rating Overrides -                                  * 
     *                   work.all_rating_overrides                              * 
     * Target Table:    Join - work.all_closed_with_party_nos                   * 
     *==========================================================================*/ 
    
    proc datasets lib = work nolist nowarn memtype = (data view);
       delete all_closed_with_party_nos;
    quit;
    
    proc sql;
       create table work.all_closed_with_party_nos as
       select
          all_closed.CASE_RK,
          all_closed.VALID_FROM_DTTM,
          all_closed.VALID_TO_DTTM,
          all_closed.CASE_ID,
          all_closed.SOURCE_SYSTEM_CD,
          all_closed.CASE_TYPE_CD,
          all_closed.CASE_CATEGORY_CD,
          all_closed.CASE_SUBCATEGORY_CD,
          all_closed.CASE_STATUS_CD,
          all_closed.CASE_DISPOSITION_CD,
          all_closed.OPEN_DTTM,
          all_closed.REOPEN_DTTM,
          all_closed.CLOSE_DTTM,
          all_closed.CREATE_DTTM,
          all_rating_overrides.RATING_OVERRIDE
       from
          work.all_closed left join 
          work.all_rating_overrides
             on
             (
                all_closed.CASE_RK = all_rating_overrides.CASE_RK
                and all_closed.VALID_FROM_DTTM = all_rating_overrides.VALID_FROM_DTTM
             )
       ;
    quit;
        
    %fcf_rcset(&sqlrc); 
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create ALL_CLOSED_W_PARTY_NO;
        %fcf_put_error;
        %goto MACRO_END;
    %end;
        
    /*==========================================================================* 
     * Step:            Join                                                    * 
     * Transform:       Join                                                    * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Tables:   Join - work.all_closed_with_party_nos                   * 
     *                  All Party Numbers -                                     * 
     *                   work.all_party_nos                                     * 
     * Target Table:    All closed cases with rating                            * 
     *                  overrides and party numbers -                           * 
     *                   work.all_closed_with_ro_no                             * 
     *==========================================================================*/ 
    
    proc datasets lib = work nolist nowarn memtype = (data view);
       delete all_closed_with_ro_no;
    quit;
    
    proc sql;
       create table work.all_closed_with_ro_no as
       select
          all_closed_with_party_nos.CASE_RK,
          all_closed_with_party_nos.VALID_FROM_DTTM,
          all_closed_with_party_nos.VALID_TO_DTTM,
          all_closed_with_party_nos.CASE_ID,
          all_closed_with_party_nos.SOURCE_SYSTEM_CD,
          all_closed_with_party_nos.CASE_TYPE_CD,
          all_closed_with_party_nos.CASE_CATEGORY_CD,
          all_closed_with_party_nos.CASE_SUBCATEGORY_CD,
          all_closed_with_party_nos.CASE_STATUS_CD,
          all_closed_with_party_nos.CASE_DISPOSITION_CD,
          all_closed_with_party_nos.OPEN_DTTM,
          all_closed_with_party_nos.REOPEN_DTTM,
          all_closed_with_party_nos.CLOSE_DTTM,
          all_closed_with_party_nos.CREATE_DTTM,
          all_closed_with_party_nos.RATING_OVERRIDE,
          all_party_nos.PRIMARY_ENTITY_NUMBER as PARTY_NUMBER
       from
          work.all_closed_with_party_nos left join 
          work.all_party_nos
             on
             (
                all_closed_with_party_nos.CASE_RK = all_party_nos.CASE_RK
                and all_closed_with_party_nos.VALID_FROM_DTTM = all_party_nos.VALID_FROM_DTTM
             )
       ;
    quit;
    
    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create ALL_CLOSED_W_RO_NO;
        %fcf_put_error;
        %goto MACRO_END;
    %end; 
    
    /*==========================================================================* 
     * Step:            Join                                                    * 
     * Transform:       Join                                                    * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Tables:   All closed cases with rating                            * 
     *                  overrides and party numbers -                           * 
     *                   work.all_closed_with_ro_no                             * 
     *                  FSK_CDD_SCORING_EVENT -                                 * 
     *                   &db_kc_libname..FSK_CDD_SCORING_EVENT                  * 
     * Target Table:    Join - work.all_closed_with_ro_no_sc                    * 
     *==========================================================================*/ 
        
    proc datasets lib = work nolist nowarn memtype = (data view);
       delete all_closed_with_ro_no_sc;
    quit;
    
    proc sql;
       create view work.all_closed_with_ro_no_sc as
       select
          all_closed_with_ro_no.CASE_RK,
          all_closed_with_ro_no.VALID_FROM_DTTM,
          all_closed_with_ro_no.VALID_TO_DTTM,
          all_closed_with_ro_no.CASE_ID,
          all_closed_with_ro_no.SOURCE_SYSTEM_CD,
          all_closed_with_ro_no.CASE_TYPE_CD,
          all_closed_with_ro_no.CASE_CATEGORY_CD,
          all_closed_with_ro_no.CASE_SUBCATEGORY_CD,
          all_closed_with_ro_no.CASE_STATUS_CD,
          all_closed_with_ro_no.CASE_DISPOSITION_CD,
          all_closed_with_ro_no.OPEN_DTTM,
          all_closed_with_ro_no.REOPEN_DTTM,
          all_closed_with_ro_no.CLOSE_DTTM,
          all_closed_with_ro_no.CREATE_DTTM,
          all_closed_with_ro_no.RATING_OVERRIDE,
          all_closed_with_ro_no.PARTY_NUMBER,
          FSK_CDD_SCORING_EVENT.SCORE_CLASSIFICATION_CODE
       from
          work.all_closed_with_ro_no left join 
          &db_kc_libname..FSK_CDD_SCORING_EVENT
             on
             (
                all_closed_with_ro_no.CASE_ID = FSK_CDD_SCORING_EVENT.CDD_SCORING_EVENT_KEY
             )
       ;
    quit;
        
    %fcf_rcset(&sqlrc); 
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create ALL_CLOSED_W_RO_NO_SC;
        %fcf_put_error;
        %goto MACRO_END;
    %end;
        
    /*==========================================================================* 
     * Step:            Get all closed Cases from KC                            * 
     * Transform:       Extract                                                 * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    FSK_CDD_SCORING_EVENT -                                 * 
     *                   &db_kc_libname..FSK_CDD_SCORING_EVENT                  * 
     * Target Table:    All closed cases from KC -                              * 
     *                   work.all_closed_from_kc                                * 
     *==========================================================================*/ 
    
    %let SOURCEDS = &db_kc_libname..FSK_CDD_SCORING_EVENT;
    
    proc datasets lib = work nolist nowarn memtype = (data view);
       delete all_closed_from_kc;
    quit;
    
    proc sql;
       create table work.all_closed_from_kc as
          select
             CDD_SCORING_EVENT_KEY,
             SEGMENT_ID,
             PRIMARY_ENTITY_KEY,
             PRIMARY_ENTITY_NUMBER,
             PRIMARY_ENTITY_LEVEL_CODE,
             CDD_RULE_ID,
             VERSION_NO,
             RULE_SCORE,
             SCORE_CLASSIFICATION_CODE,
             EVENT_DATE,
             AUTO_HIGH_IND,
             INVESTIGATION_IND,
             INVESTIGATION_ACTION_CODE,
             CATEGORY_CODE,
             SCORE_TYPE_CODE,
             INVESTIGATION_TYPE_CD,
             INVESTIGATION_CATEGORY_CD,
             INVESTIGATION_SUBCATEGORY_CD,
             INVESTIGATION_RK,
             INVESTIGATION_STATUS_CD,
             INVESTIGATION_OBJECT_CODE
       from &SOURCEDS
          where INVESTIGATION_STATUS_CD = 'C'  |  INVESTIGATION_STATUS_CD = 'T'
       ;
    quit;
    
    %let SOURCEDS = work.all_closed_from_kc;
    
    %fcf_rcset(&sqlrc); 
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to retrieve all closed CDD Cases from Knowledge Center;
        %fcf_put_error;
        %goto MACRO_END;
    %end;
    
    /*==========================================================================* 
     * Step:            Join                                                    * 
     * Transform:       Join                                                    * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Tables:   All closed cases from KC -                              * 
     *                   work.all_closed_from_kc                                * 
     *                  Join - work.all_closed_with_ro_no_sc                    * 
     * Target Table:    All unclosed cases - work.all_unclosed                  * 
     *==========================================================================*/ 
    
    proc datasets lib = iem nolist nowarn memtype = (data view);
       delete all_unclosed;
    quit;
    
    proc sql;
       create table work.all_unclosed as
       select
          all_closed_with_ro_no_sc.CASE_RK,
          all_closed_with_ro_no_sc.VALID_FROM_DTTM,
          all_closed_with_ro_no_sc.VALID_TO_DTTM,
          all_closed_with_ro_no_sc.CASE_ID,
          all_closed_with_ro_no_sc.SOURCE_SYSTEM_CD,
          all_closed_with_ro_no_sc.CASE_TYPE_CD,
          all_closed_with_ro_no_sc.CASE_CATEGORY_CD,
          all_closed_with_ro_no_sc.CASE_SUBCATEGORY_CD,
          all_closed_with_ro_no_sc.CASE_STATUS_CD,
          all_closed_with_ro_no_sc.CASE_DISPOSITION_CD,
          all_closed_with_ro_no_sc.OPEN_DTTM,
          all_closed_with_ro_no_sc.REOPEN_DTTM,
          all_closed_with_ro_no_sc.CLOSE_DTTM,
          all_closed_with_ro_no_sc.CREATE_DTTM,
          all_closed_with_ro_no_sc.RATING_OVERRIDE,
          all_closed_with_ro_no_sc.PARTY_NUMBER,
          all_closed_with_ro_no_sc.SCORE_CLASSIFICATION_CODE
       from
          work.all_closed_with_ro_no_sc left join 
          work.all_closed_from_kc
             on
             (
                all_closed_with_ro_no_sc.CASE_ID = all_closed_from_kc.CDD_SCORING_EVENT_KEY
             )
       where
          all_closed_with_ro_no_sc.CASE_STATUS_CD <> all_closed_from_kc.INVESTIGATION_STATUS_CD
       ;
    quit;
        
    %fcf_rcset(&sqlrc); 
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create ALL_UNCLOSED;
        %fcf_put_error;
        %goto MACRO_END;
    %end;
            
    /*==========================================================================* 
     * Step:            Join                                                    * 
     * Transform:       Join                                                    * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Tables:   All unclosed cases - work.all_unclosed                  * 
     *                  FSK_CDD_ENTITY_FACT -                                   * 
     *                   &db_kc_libname..FSK_CDD_ENTITY_FACT                    * 
     * Target Table:    All unclosed with facts -                               * 
     *                   work.all_unclosed_w_facts                              * 
     *==========================================================================*/ 
        
    proc datasets lib = work nolist nowarn memtype = (data view);
       delete all_unclosed_w_facts;
    quit;
        
    proc sql;
       create table work.all_unclosed_w_facts as
       select
          all_unclosed.CASE_RK,
          all_unclosed.VALID_FROM_DTTM,
          all_unclosed.VALID_TO_DTTM,
          all_unclosed.CASE_ID,
          all_unclosed.SOURCE_SYSTEM_CD,
          all_unclosed.CASE_TYPE_CD,
          all_unclosed.CASE_CATEGORY_CD,
          all_unclosed.CASE_SUBCATEGORY_CD,
          all_unclosed.CASE_STATUS_CD,
          all_unclosed.CASE_DISPOSITION_CD,
          all_unclosed.OPEN_DTTM,
          all_unclosed.REOPEN_DTTM,
          all_unclosed.CLOSE_DTTM,
          all_unclosed.CREATE_DTTM,
          all_unclosed.RATING_OVERRIDE,
          all_unclosed.PARTY_NUMBER,
          all_unclosed.SCORE_CLASSIFICATION_CODE,
          FSK_CDD_ENTITY_FACT.PRIMARY_ENTITY_NUMBER,
          FSK_CDD_ENTITY_FACT.PRIMARY_ENTITY_LEVEL_CODE,
          FSK_CDD_ENTITY_FACT.SEGMENT_ID,
          FSK_CDD_ENTITY_FACT.LAST_CHANGE_DATE,
          . as NEXT_REVIEW_DATE,
          FSK_CDD_ENTITY_FACT.OVERRIDE_SCORE_CLASS_CODE,
          FSK_CDD_ENTITY_FACT.FINAL_DISPOSITION_CODE,
          FSK_CDD_ENTITY_FACT.LAST_REVIEW_DATE,
          FSK_CDD_ENTITY_FACT.LAST_SCORING_EVENT_KEY,
          FSK_CDD_ENTITY_FACT.LAST_REVIEW_SCORING_EVENT_KEY,
          FSK_CDD_ENTITY_FACT.CURR_REVIEW_SCORING_EVENT_KEY
       from
          work.all_unclosed, 
          &db_kc_libname..FSK_CDD_ENTITY_FACT
       where
          all_unclosed.PARTY_NUMBER = FSK_CDD_ENTITY_FACT.PRIMARY_ENTITY_NUMBER
       ;
    quit;
        
    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create ALL_UNCLOSED_W_FACTS;
        %fcf_put_error;
        %goto MACRO_END;
    %end; 
    
    data work.all_unclosed_w_facts;
    set work.all_unclosed_w_facts;
        %fcf_iem_cdd_calc_nrd;
    run;
    
    %fcf_rcset(&syserr); 
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to calculate NEXT_REVIEW_DATE for CDD Cases;
        %fcf_put_error;
        %goto MACRO_END;
    %end;
    
    /*==========================================================================* 
     * Step:            Update                                                  * 
     * Transform:       Update                                                  * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    All unclosed with facts -                               * 
     *                   work.all_unclosed_w_facts                              * 
     * Target Table:    FSK_CDD_ENTITY_FACT -                                   * 
     *                   &db_kc_libname..FSK_CDD_ENTITY_FACT                    *
     *==========================================================================*/ 
    
    %let SOURCEDS = %nrquote(work.all_unclosed_w_facts); 
    
	proc sql;
		connect to &dbflavor as &dbflavor(authdomain="&segkcauthdomain."
			&segKCDBConnOpts);
		execute (delete from &segKCSchema..FSK_TEMP_TABLE) by &dbflavor;
		execute (commit) by &dbflavor;
		disconnect from &dbflavor;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to empty FSK_TEMP_TABLE;
		%fcf_put_error;
		%goto MACRO_END;
	%end;

	data fsk_temp_table;
		if 0 then set &seg_kc_libname..fsk_temp_table;
		set &SOURCEDS. (keep=PRIMARY_ENTITY_NUMBER CASE_DISPOSITION_CD RATING_OVERRIDE
						CURR_REVIEW_SCORING_EVENT_KEY CLOSE_DTTM NEXT_REVIEW_DATE);

		TEMP_CHAR_1_VALUE=PRIMARY_ENTITY_NUMBER;
		TEMP_CHAR_2_VALUE=CASE_DISPOSITION_CD;
		TEMP_CHAR_3_VALUE=RATING_OVERRIDE;
		TEMP_NUM_1_VALUE=CURR_REVIEW_SCORING_EVENT_KEY;
		TEMP_DATE_1_VALUE=CLOSE_DTTM;
		TEMP_DATE_2_VALUE=NEXT_REVIEW_DATE;

		drop 	PRIMARY_ENTITY_NUMBER CASE_DISPOSITION_CD RATING_OVERRIDE
				CURR_REVIEW_SCORING_EVENT_KEY CLOSE_DTTM NEXT_REVIEW_DATE;
	run;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to create staging dataset for FSK_TEMP_TABLE load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;

	proc append	base=&seg_kc_libname..fsk_temp_table data=work.fsk_temp_table force;
	quit;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to append to FSK_TEMP_TABLE;
		%fcf_put_error;
		%goto MACRO_END;
	%end;

	proc sql;
		connect to &dbflavor as &dbflavor (authdomain="&segkcauthdomain." &segKCDBConnOpts);
		execute (
			merge into fsk_cdd_entity_fact ent_fct using (
				select	TEMP_CHAR_1_VALUE, 
						TEMP_CHAR_2_VALUE, 
						TEMP_CHAR_3_VALUE, 	
						TEMP_NUM_1_VALUE,
						TEMP_DATE_1_VALUE,
						TEMP_DATE_2_VALUE
				from 	FSK_TEMP_TABLE
			) fsk_tmp
			on (
				fsk_tmp.TEMP_CHAR_1_VALUE = ent_fct.primary_entity_number 
			)
			when matched then update set 
				ent_fct.CURR_REVIEW_SCORING_EVENT_KEY = null,
				ent_fct.LAST_REVIEW_SCORING_EVENT_KEY = fsk_tmp.TEMP_NUM_1_VALUE,
				ent_fct.LAST_REVIEW_DATE = fsk_tmp.TEMP_DATE_1_VALUE,
				ent_fct.FINAL_DISPOSITION_CODE = fsk_tmp.TEMP_CHAR_2_VALUE,
				ent_fct.OVERRIDE_SCORE_CLASS_CODE = fsk_tmp.TEMP_CHAR_3_VALUE,
				ent_fct.NEXT_REVIEW_DATE = fsk_tmp.TEMP_DATE_2_VALUE,
				ent_fct.LAST_CHANGE_DATE = fsk_tmp.TEMP_DATE_1_VALUE
				
		) by &dbflavor;
		disconnect from &dbflavor;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to update closed case information in FSK_CDD_ENTITY_FACT;
		%fcf_put_error;
		%goto MACRO_END;
	%end;


    /*==========================================================================* 
     * Step:            Update                                                  * 
     * Transform:       Update                                                  * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    All unclosed with facts -                               * 
     *                   work.all_unclosed_w_facts                              * 
     * Target Table:    FSK_CDD_SCORING EVENT -                                 * 
     *                   &db_kc_libname..FSK_CDD_SCORING_EVENT                  *
     *==========================================================================*/ 
    
    %let SOURCEDS = %nrquote(work.all_unclosed_w_facts); 
    
	proc sql;
		connect to &dbflavor as &dbflavor(authdomain="&segkcauthdomain."
			&segKCDBConnOpts);
		execute (delete from &segKCSchema..FSK_TEMP_TABLE) by &dbflavor;
		execute (commit) by &dbflavor;
		disconnect from &dbflavor;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to empty FSK_TEMP_TABLE;
		%fcf_put_error;
		%goto MACRO_END;
	%end;

	data fsk_temp_table;
		if 0 then set &seg_kc_libname..fsk_temp_table;
		set &SOURCEDS. (keep=CASE_ID CASE_STATUS_CD);

		TEMP_NUM_1_VALUE=CASE_ID;
		TEMP_CHAR_1_VALUE=CASE_STATUS_CD;

		drop CASE_ID CASE_STATUS_CD;
	run;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to create staging dataset for FSK_TEMP_TABLE load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;

	proc append	base=&seg_kc_libname..fsk_temp_table data=work.fsk_temp_table force;
	quit;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to append to FSK_TEMP_TABLE;
		%fcf_put_error;
		%goto MACRO_END;
	%end;

	proc sql;
		connect to &dbflavor as &dbflavor (authdomain="&segkcauthdomain." &segKCDBConnOpts);
		execute (
			merge into fsk_cdd_scoring_event sc_e using (
				select	TEMP_CHAR_1_VALUE,
						TEMP_NUM_1_VALUE
				from 	FSK_TEMP_TABLE
			) fsk_tmp
			on (
				fsk_tmp.TEMP_NUM_1_VALUE = sc_e.CDD_SCORING_EVENT_KEY 
			)
			when matched then update set 
				sc_e.INVESTIGATION_STATUS_CD = fsk_tmp.TEMP_CHAR_1_VALUE
		) by &dbflavor;
		disconnect from &dbflavor;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to update closed case information in FSK_CDD_SCORING_EVENT;
		%fcf_put_error;
		%goto MACRO_END;
	%end;


    /*==========================================================================* 
     * Step:            Push dates back into ECM                                * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    All unclosed with facts -                               * 
     *                   work.all_unclosed_w_facts                              * 
     * Target Table:    Update CDD Cases - iem.cdd_upd_cases                    * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.all_unclosed_w_facts); 

    proc datasets lib = iem nolist nowarn memtype = (data view);
        delete cdd_upd_case;
    quit;

	data iem.cdd_upd_case;
		if 0 then set ecm_db.case_live;
		set &SOURCEDS. (keep= CASE_RK NEXT_REVIEW_DATE CLOSE_DTTM);
		CASE_RK = CASE_RK;
		NEXT_REVIEW_DATE = NEXT_REVIEW_DATE;
		LAST_REVIEW_DATE = CLOSE_DTTM;
		drop CLOSE_DTTM;
	run;

    %fcf_rcset(&syserr);
    
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create IEM.CDD_UPD_CASE;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

	%if %fcf_get_observation_count(table=iem.cdd_upd_case,SASDATASET=Y) lt 1 %then %goto skip_update;
    	%fcf_iem_cdd_update_case(ds=iem.cdd_upd_case);
    %skip_update:

%MACRO_END:

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
        %fcf_execute_macro(fcf_error);
    %else %do;
        %fcf_execute_macro(fcf_success);
    %end;

%mend;

