/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_iem_fb_alerts_risks.sas
| PRODUCT: SAS Financial Crimes Foundation
| SYSTEM:  Windows, Unix
| DESC:    ECM to FCF feedback of Alert, Alert Reminder, and Risk
|          Assessment updates.
+------------------------------------------------------------------------------+
| USAGE:   %fcf_iem_fb_alerts_risks;
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_iem_fb_alerts_risks;

    /*------------------------------------------
     * Set error code to 0 and message to blank
     *------------------------------------------*/

    %let trans_rc = 0;
    %let _fcfmsg_=;

    /*==========================================================================* 
     * Step:            Extract                                                 * 
     * Transform:       Extract                                                 * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    INCIDENT_LIVE - ecm_db.INCIDENT_LIVE                    * 
     * Target Table:    Latest Alerts and Risk Assessments -                    * 
     *                   work.latest_alts_rsks                                  * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(ecm_db.INCIDENT_LIVE);

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete latest_alts_rsks;
    quit;

    proc sql;
        create view work.latest_alts_rsks as
            select
                (input((substrn(INCIDENT_ID,5)),best.)) as INCIDENT_ID,
                INCIDENT_RK,
                VALID_FROM_DTTM,
                CASE_RK,
                SOURCE_SYSTEM_CD,
                INVESTIGATOR_USER_ID,
                INCIDENT_DISPOSITION_CD,
                CLOSE_DTTM,
                INCIDENT_TYPE_CD,
                INCIDENT_CATEGORY_CD,
                INCIDENT_SUBCATEGORY_CD
            from &SOURCEDS
                where date() - datepart(VALID_FROM_DTTM) <= 100 & SOURCE_SYSTEM_CD="&segKCschema" & (INCIDENT_TYPE_CD = "AMLALT"  | INCIDENT_TYPE_CD  = "AMLRISK") & (INCIDENT_ID like 'ALT-%' | INCIDENT_ID like 'MAL-%' | INCIDENT_ID like 'RAS-%' | INCIDENT_ID like 'MRA%');
        ;
    quit;

    %fcf_rcset(&sqlrc);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to retrieve latest AML Alert and Risk Assessment versions for updates;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Data Validation                                         * 
     * Transform:       Data Validation                                         * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Latest Alerts and Risk Assessments -                    * 
     *                   work.latest_alts_rsks                                  * 
     * Target Table:    Fixed Disposition - work.fixed_disp                     * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(work.latest_alts_rsks);

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete fixed_disp;
    quit;

    data _null_;
        call symput('current_dt',put(datetime(),datetime22.));
    run;

    data work.fixed_disp
        (keep = INCIDENT_ID INCIDENT_RK VALID_FROM_DTTM CASE_RK 
        SOURCE_SYSTEM_CD INVESTIGATOR_USER_ID INCIDENT_DISPOSITION_CD 
        CLOSE_DTTM INCIDENT_TYPE_CD INCIDENT_CATEGORY_CD INCIDENT_SUBCATEGORY_CD);
        set &SOURCEDS end=eof;

        /*---- Custom Validation: Assigned to case  ----*/
        if (CASE_RK ^= .) then
            do;
                INCIDENT_DISPOSITION_CD = 'CLC';
                CLOSE_DTTM = "&current_dt"dt; 
            end;  /* if (CASE_RK ^= '') */
        else
            do;
            end;  /* else */

        output work.fixed_disp;

        if eof then
            do;
            end;
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to set Disposition code for Incidents assigned to Cases;

        %fcf_put_error;
        %goto MACRO_END;
    %end;


    /*==========================================================================* 
     * Step:            Get Suppression End Dates                               * 
     * Transform:       Extract                                                 * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    INCIDENT_UDF_DATE_VALUE -                               * 
     *                   ecm_db.INCIDENT_UDF_DATE_VALUE                         * 
     * Target Table:    Suppression End dates -                                 * 
     *                   work.supp_end_dates                                    * 
     *==========================================================================*/ 
    
    %let SOURCEDS = %nrquote(ecm_db.INCIDENT_UDF_DATE_VALUE); 
    
    proc datasets lib = work nolist nowarn memtype = (data view);
       delete supp_end_dates;
    quit;
    
    proc sql;
       create view work.supp_end_dates as
          select
             INCIDENT_RK,
             VALID_FROM_DTTM,
             UDF_TABLE_NM,
             UDF_NM,
             ROW_NO,
             UDF_VALUE
       from &SOURCEDS
          where INCIDENT_UDF_DATE_VALUE.UDF_NM = "X_SUPPRESSION_END_DATE"
       ;
    quit;
    
    %let SOURCEDS = work.supp_end_dates;
    
    %fcf_rcset(&sqlrc);
    
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to get Suppression End values from INCIDENT_UDF_DATE_VALUE;
    
        %fcf_put_error;
        %goto MACRO_END;
    %end;
    
    /*==========================================================================* 
     * Step:            Get Alert Reminder Dates                                * 
     * Transform:       Extract                                                 * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    INCIDENT_UDF_DATE_VALUE -                               * 
     *                   ecm_db.INCIDENT_UDF_DATE_VALUE                         * 
     * Target Table:    Alert Reminder dates -                                  * 
     *                   work.alt_remind_dates                                  * 
     *==========================================================================*/ 
    
    %let SOURCEDS = %nrquote(ecm_db.INCIDENT_UDF_DATE_VALUE); 
    
    proc datasets lib = work nolist nowarn memtype = (data view);
       delete alt_remind_dates;
    quit;
    
    proc sql;
       create view work.alt_remind_dates as
          select
             INCIDENT_RK,
             VALID_FROM_DTTM,
             UDF_TABLE_NM,
             UDF_NM,
             ROW_NO,
             UDF_VALUE
       from &SOURCEDS
          where INCIDENT_UDF_DATE_VALUE.UDF_NM = "ALT_REMINDER_DATE"
       ;
    quit;
    
    %let SOURCEDS = work.alt_remind_dates;
    
    %fcf_rcset(&sqlrc);
    
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to get Alert Reminder values from INCIDENT_UDF_DATE_VALUE;
    
        %fcf_put_error;
        %goto MACRO_END;
    %end;
    
    /*==========================================================================* 
     * Step:            Get Closing Reasons                                     * 
     * Transform:       Extract                                                 * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    INCIDENT_UDF_CHAR_VALUE -                               * 
     *                   ecm_db.INCIDENT_UDF_CHAR_VALUE                         * 
     * Target Table:    Closing Reasons - work.close_reasons                    * 
     *==========================================================================*/ 
    
    %let SOURCEDS = %nrquote(ecm_db.INCIDENT_UDF_CHAR_VALUE); 
    
    proc datasets lib = work nolist nowarn memtype = (data view);
       delete close_reasons;
    quit;
    
    proc sql;
       create view work.close_reasons as
          select
             INCIDENT_RK,
             VALID_FROM_DTTM,
             UDF_TABLE_NM,
             UDF_NM,
             ROW_NO,
             UDF_VALUE
       from &SOURCEDS
          where INCIDENT_UDF_CHAR_VALUE.UDF_NM = "X_REASON"
       ;
    quit;
    
    %let SOURCEDS = work.close_reasons;
    
    %fcf_rcset(&sqlrc);
    
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to get Reason values from INCIDENT_UDF_CHAR_VALUE;
    
        %fcf_put_error;
        %goto MACRO_END;
    %end;
    
    /*==========================================================================* 
     * Step:            Join                                                    * 
     * Transform:       Join                                                    * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Tables:   Fixed Disposition - work.fixed_disp                     * 
     *                  Suppression End dates -                                 * 
     *                   work.supp_end_dates                                    * 
     * Target Table:    Latest Alerts and Risk Assessments                      * 
     *                   with Dates - work.alts_rsks_w_sup                      * 
     *==========================================================================*/ 
    
    proc datasets lib = work nolist nowarn memtype = (data view);
       delete alts_rsks_w_sup;
    quit;
    
    proc sql;
       create view work.alts_rsks_w_sup as
       select
          fixed_disp.INCIDENT_ID,
          fixed_disp.INCIDENT_RK,
          fixed_disp.VALID_FROM_DTTM,
          fixed_disp.INVESTIGATOR_USER_ID,
          fixed_disp.INCIDENT_DISPOSITION_CD,
          fixed_disp.CLOSE_DTTM,
          fixed_disp.SOURCE_SYSTEM_CD,
          fixed_disp.INCIDENT_TYPE_CD,
          fixed_disp.INCIDENT_CATEGORY_CD,
          fixed_disp.INCIDENT_SUBCATEGORY_CD,
          supp_end_dates.UDF_VALUE as SUPP_END_DT
       from
          work.fixed_disp left join 
          work.supp_end_dates
             on
             (
                fixed_disp.INCIDENT_RK = supp_end_dates.INCIDENT_RK
                and fixed_disp.VALID_FROM_DTTM = supp_end_dates.VALID_FROM_DTTM
             )
       ;
    quit;
    
    %fcf_rcset(&sqlrc);
    
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to join ALT and RAS updates with Suppression End dates;
    
        %fcf_put_error;
        %goto MACRO_END;
    %end;
    
    /*==========================================================================* 
     * Step:            Join                                                    * 
     * Transform:       Join                                                    * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Tables:   Latest Alerts and Risk Assessments                      * 
     *                   with Dates - work.alts_rsks_w_sup                      * 
     *                  Alert Reminder dates -                                  * 
     *                   work.alt_remind_dates                                  * 
     * Target Table:    Latest Alerts and Risk Assessments                      * 
     *                  with Dates -                                            * 
     *                   work.alts_rsks_w_sup_rmd                               * 
     *==========================================================================*/ 
    
    proc datasets lib = work nolist nowarn memtype = (data view);
       delete alts_rsks_w_sup_rmd;
    quit;
    
    proc sql;
       create view work.alts_rsks_w_sup_rmd as
       select
          alts_rsks_w_sup.INCIDENT_ID,
          alts_rsks_w_sup.INCIDENT_RK,
          alts_rsks_w_sup.VALID_FROM_DTTM,
          alts_rsks_w_sup.INVESTIGATOR_USER_ID,
          alts_rsks_w_sup.INCIDENT_DISPOSITION_CD,
          alts_rsks_w_sup.CLOSE_DTTM,
          alts_rsks_w_sup.SOURCE_SYSTEM_CD,
          alts_rsks_w_sup.INCIDENT_TYPE_CD,
          alts_rsks_w_sup.INCIDENT_CATEGORY_CD,
          alts_rsks_w_sup.INCIDENT_SUBCATEGORY_CD,
          alts_rsks_w_sup.SUPP_END_DT,
          alt_remind_dates.UDF_VALUE as ALT_REMINDER_DT
       from
          work.alts_rsks_w_sup left join 
          work.alt_remind_dates
             on
             (
                alts_rsks_w_sup.INCIDENT_RK = alt_remind_dates.INCIDENT_RK
                and alts_rsks_w_sup.VALID_FROM_DTTM = alt_remind_dates.VALID_FROM_DTTM
             )
       ;
    quit;
    
    %fcf_rcset(&sqlrc);
    
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to join ALT and RAS updates with Alert Reminder dates;
    
        %fcf_put_error;
        %goto MACRO_END;
    %end;
    
    /*==========================================================================* 
     * Step:            Join                                                    * 
     * Transform:       Join                                                    * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Tables:   Latest Alerts and Risk Assessments                      * 
     *                  with Dates -                                            * 
     *                   work.alts_rsks_w_sup_rmd                               * 
     *                  Closing Reasons - work.close_reasons                    * 
     * Target Table:    Latest Alerts and Risk Assessments                      * 
     *                  with Dates and Reason -                                 * 
     *                   work.alts_rsks_w_sup_rmd_rsn                           * 
     *==========================================================================*/ 
    
    proc datasets lib = work nolist nowarn memtype = (data view);
       delete alts_rsks_w_sup_rmd_rsn;
    quit;
    
    proc sql;
       create view work.alts_rsks_w_sup_rmd_rsn as
       select
          alts_rsks_w_sup_rmd.INCIDENT_ID,
          alts_rsks_w_sup_rmd.INCIDENT_RK,
          alts_rsks_w_sup_rmd.VALID_FROM_DTTM,
          alts_rsks_w_sup_rmd.INVESTIGATOR_USER_ID,
          alts_rsks_w_sup_rmd.INCIDENT_DISPOSITION_CD,
          alts_rsks_w_sup_rmd.CLOSE_DTTM,
          alts_rsks_w_sup_rmd.SOURCE_SYSTEM_CD,
          alts_rsks_w_sup_rmd.INCIDENT_TYPE_CD,
          alts_rsks_w_sup_rmd.INCIDENT_CATEGORY_CD,
          alts_rsks_w_sup_rmd.INCIDENT_SUBCATEGORY_CD,
          alts_rsks_w_sup_rmd.SUPP_END_DT,
          alts_rsks_w_sup_rmd.ALT_REMINDER_DT,
          close_reasons.UDF_VALUE as EVENT_DESC
       from
          work.alts_rsks_w_sup_rmd left join 
          work.close_reasons
             on
             (
                alts_rsks_w_sup_rmd.INCIDENT_RK = close_reasons.INCIDENT_RK
                and alts_rsks_w_sup_rmd.VALID_FROM_DTTM = close_reasons.VALID_FROM_DTTM
             )
       ;
    quit;
    
    %fcf_rcset(&sqlrc);
    
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to join ALT and RAS updates with Close Reasons;
    
        %fcf_put_error;
        %goto MACRO_END;
    %end;


    /*==========================================================================* 
     * Step:            Splitter                                                * 
     * Transform:       Splitter                                                * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Latest Alerts and Risk Assessments                      * 
     *                  with Dates and Reason -                                 * 
     *                   work.alts_rsks_w_sup_rmd_rsn                           * 
     * Target Tables:   Alert Updates - work.alert_updates                      * 
     *                  Risk Assessment Updates -                               * 
     *                   work.risk_assessment_updates                           * 
     *                  Alert Reminder Updates -                                * 
     *                   work.alert_reminder_updates                            * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(work.alts_rsks_w_sup_rmd_rsn);

    proc datasets lib=work nolist nowarn memtype = (data view);
        delete alert_updates;
        delete risk_assessment_updates;
        delete alert_reminder_updates;
    quit;

    data work.alert_updates
        work.risk_assessment_updates
        work.alert_reminder_updates;
        set &SOURCEDS;

        if INCIDENT_TYPE_CD  = "AMLALT" then
            output work.alert_updates;

        if INCIDENT_TYPE_CD  = "AMLRISK" then
            output work.risk_assessment_updates;

        if ALT_REMINDER_DT ^= . then
            output work.alert_reminder_updates;
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to split ALT and RAS updates;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

    proc sql;
        create table work.temp_map as
            select
                INCIDENT_ID,
                INVESTIGATOR_USER_ID,
                INCIDENT_DISPOSITION_CD,
                CLOSE_DTTM,
                SUPP_END_DT,
                SOURCE_SYSTEM_CD,
                EVENT_DESC
            from work.alert_updates
        ;
    quit;

    %let SOURCEDS = work.temp_map;

    data work.alert_updates;
        set work.temp_map;
    run;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

    proc sql;
        create table work.temp_map as
            select
                INCIDENT_ID,
                INVESTIGATOR_USER_ID,
                INCIDENT_DISPOSITION_CD,
                SOURCE_SYSTEM_CD
            from work.risk_assessment_updates
        ;
    quit;

    %let SOURCEDS = work.temp_map;

    data work.risk_assessment_updates;
        set work.temp_map;
    run;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

    proc sql;
        create table work.temp_map as
            select
                INCIDENT_ID,
                INVESTIGATOR_USER_ID,
                ALT_REMINDER_DT,
                SOURCE_SYSTEM_CD
            from work.alert_reminder_updates
        ;
    quit;

    %let SOURCEDS = work.temp_map;

    data work.alert_reminder_updates;
        set work.temp_map;
    run;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

    /*==========================================================================* 
     * Step:            Update                                                  * 
     * Transform:       Update                                                  * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Alert Updates - work.alert_updates                      * 
     * Target Table:    FSK_ALERT - &db_kc_libname..FSK_ALERT                             * 
     *==========================================================================*/

    proc sql;
        update &db_kc_libname..FSK_ALERT
            set 
                ALERT_STATUS_CODE= (
                select
                    al_upd.INCIDENT_DISPOSITION_CD
                from
                    work.alert_updates as al_upd
                where
                    FSK_ALERT.ALERT_ID = al_upd.INCIDENT_ID
                    and FSK_ALERT.SEGMENT_ID = al_upd.SOURCE_SYSTEM_CD
                ),


                OWNER_USER_LONG_ID= (
                select
                    al_upd.INVESTIGATOR_USER_ID
                from
                    work.alert_updates as al_upd
                where
                    FSK_ALERT.ALERT_ID = al_upd.INCIDENT_ID
                    and FSK_ALERT.SEGMENT_ID = al_upd.SOURCE_SYSTEM_CD
                ),

                SUPPRESSION_END_DATE= (
                select
                    al_upd.SUPP_END_DT
                from
                    work.alert_updates as al_upd
                where
                    FSK_ALERT.ALERT_ID = al_upd.INCIDENT_ID
                    and FSK_ALERT.SEGMENT_ID = al_upd.SOURCE_SYSTEM_CD
                )

            where ALERT_ID in (select INCIDENT_ID from work.Alert_updates)
            and SEGMENT_ID in (select SOURCE_SYSTEM_CD from work.Alert_updates)
        ;
    quit;

    %fcf_rcset(&sqlrc);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to update Alerts in FSK_ALERT;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Extract                                                 * 
     * Transform:       Extract                                                 * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Alert Updates - work.alert_updates                      * 
     * Target Table:    Alert Events - work.alert_events                        * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(work.alert_updates);

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete alert_events;
    quit;

    proc sql;
        create view work.alert_events as
            select
                INCIDENT_ID,
                INVESTIGATOR_USER_ID,
                INCIDENT_DISPOSITION_CD,
                CLOSE_DTTM,
                SOURCE_SYSTEM_CD,
                EVENT_DESC
            from &SOURCEDS
                where INCIDENT_DISPOSITION_CD = "CLS"  | INCIDENT_DISPOSITION_CD = "CLC"
        ;
    quit;

    %let SOURCEDS = work.alert_events;

    %fcf_rcset(&sqlrc);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to get ALT updates for Close events;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Join                                                    * 
     * Transform:       Join                                                    * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Tables:   Alert Events - work.alert_events                        * 
     *                  FSK_ALERT_EVENT -                                       * 
     *                   &db_kc_libname..FSK_ALERT_EVENT                        * 
     * Target Table:    Alert Event Stage -                                     * 
     *                   iem.alert_event_stage                                  * 
     *==========================================================================*/

    proc datasets lib = iem nolist nowarn memtype = (data view);
        delete alert_event_stage;
    quit;

    proc sql;
        create table iem.alert_event_stage as
            select
                . as EVENT_ID,
                alert_events.INCIDENT_ID,
                alert_events.INVESTIGATOR_USER_ID,
                alert_events.INCIDENT_DISPOSITION_CD,
                alert_events.CLOSE_DTTM,
                alert_events.EVENT_DESC,
                alert_events.SOURCE_SYSTEM_CD
            from
                work.alert_events left join 
                &db_kc_libname..FSK_ALERT_EVENT
                on
                (
                alert_events.INCIDENT_ID = FSK_ALERT_EVENT.ALERT_ID
                and alert_events.SOURCE_SYSTEM_CD = FSK_ALERT_EVENT.SEGMENT_ID
                and alert_events.INVESTIGATOR_USER_ID = FSK_ALERT_EVENT.CREATE_USER_ID
                and alert_events.INCIDENT_DISPOSITION_CD = FSK_ALERT_EVENT.EVENT_TYPE_CODE
                and alert_events.EVENT_DESC = FSK_ALERT_EVENT.EVENT_DESCRIPTION
                    )
                where
                    FSK_ALERT_EVENT.ALERT_ID IS NULL 
        ;
    quit;

    %fcf_rcset(&sqlrc);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to get ALT updates to be appended to FSK_ALERT_EVENT;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    %let obs_count = %fcf_get_observation_count(table=iem.alert_event_stage,SASDATASET=Y);
    %let numevents = &obs_count;

    %fcf_get_sequence_ids(numIDs=&numevents);

    data iem.alert_event_stage;
        attrib event_id informat=best32. format=best32. label='EVENT_ID';
        set iem.alert_event_stage;
        event_id = symgetn('seqId'||(kstrip(put(_n_,best32.))));
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to generate primary keys for FSK_ALERT_EVENT update;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Table Loader                                            * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Alert Event Stage -                                     * 
     *                   iem.alert_event_stage                                  * 
     * Target Table:    FSK_ALERT_EVENT -                                       * 
     *                   &db_kc_libname..FSK_ALERT_EVENT                        * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(iem.alert_event_stage);

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;



    proc sql;
        create view work.temp_map as
            select
                EVENT_ID,
                SOURCE_SYSTEM_CD as SEGMENT_ID,
                INCIDENT_ID as ALERT_ID,
                INCIDENT_DISPOSITION_CD as EVENT_TYPE_CODE,
                EVENT_DESC as EVENT_DESCRIPTION,
                CLOSE_DTTM as CREATE_DATE,
                INVESTIGATOR_USER_ID as CREATE_USER_ID
            from &SOURCEDS
        ;
    quit;

    %let SOURCEDS = work.temp_map;

    proc append base = &db_kc_libname..FSK_ALERT_EVENT 
        data = &SOURCEDS force;
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to append to FSK_ALERT_EVENT for ALT updates;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

    /*==========================================================================* 
     * Step:            Update                                                  * 
     * Transform:       Update                                                  * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Tables:   Risk Assessment Updates -                               * 
     *                   work.risk_assessment_updates                           * 
     *                  FSK_RISK_ASSESSMENT -                                   * 
     *                   &db_kc_libname..FSK_RISK_ASSESSMENT                    * 
     * Target Table:    FSK_RISK_ASSESSMENT -                                   * 
     *                   &db_kc_libname..FSK_RISK_ASSESSMENT                    * 
     *==========================================================================*/

    proc sql;
        update &db_kc_libname..FSK_RISK_ASSESSMENT
            set 
                OWNER_USER_LONG_ID= (
                select
                    r_a_u.INVESTIGATOR_USER_ID
                from
                    work.risk_assessment_updates as r_a_u
                where
                    r_a_u.INCIDENT_ID = FSK_RISK_ASSESSMENT.RISK_ASSESSMENT_ID
                and r_a_u.SOURCE_SYSTEM_CD = FSK_RISK_ASSESSMENT.SEGMENT_ID
                ),
                RISK_ASSESSMENT_STATUS_CODE= (
                select
                    r_a_u.INCIDENT_DISPOSITION_CD
                from
                    work.risk_assessment_updates as r_a_u
                where
                    r_a_u.INCIDENT_ID = FSK_RISK_ASSESSMENT.RISK_ASSESSMENT_ID
                and r_a_u.SOURCE_SYSTEM_CD = FSK_RISK_ASSESSMENT.SEGMENT_ID
                )
            where risk_assessment_id IN (select INCIDENT_ID from work.risk_assessment_updates)
            and segment_id IN (select SOURCE_SYSTEM_CD from work.risk_assessment_updates)
        ;
    quit;

    %fcf_rcset(&sqlrc);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to update FSK_RISK_ASSESSMENT for RAS updates;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Join                                                    * 
     * Transform:       Join                                                    * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Tables:   Alert Reminder Updates -                                * 
     *                   work.alert_reminder_updates                            * 
     *                  FSK_ALERT_REMINDER -                                    * 
     *                   &db_kc_libname..FSK_ALERT_REMINDER                     * 
     * Target Table:    Alert Reminder Stage -                                  * 
     *                   iem.alert_reminder_stage                               * 
     *==========================================================================*/

    proc datasets lib = iem nolist nowarn memtype = (data view);
        delete alert_reminder_stage;
    quit;

    proc sql;
        create table iem.alert_reminder_stage as
            select
                alert_reminder_updates.INCIDENT_ID as ALERT_ID,
                alert_reminder_updates.INVESTIGATOR_USER_ID as USER_LONG_ID,
                alert_reminder_updates.SOURCE_SYSTEM_CD,
                alert_reminder_updates.ALT_REMINDER_DT,
                'N' as REMINDER_SENT_IND
            from
                work.alert_reminder_updates left join 
                &db_kc_libname..FSK_ALERT_REMINDER
                on
                (
                alert_reminder_updates.INCIDENT_ID = FSK_ALERT_REMINDER.ALERT_ID
                and alert_reminder_updates.SOURCE_SYSTEM_CD = FSK_ALERT_REMINDER.SEGMENT_ID
                and alert_reminder_updates.INVESTIGATOR_USER_ID = FSK_ALERT_REMINDER.USER_LONG_ID
                and alert_reminder_updates.ALT_REMINDER_DT = FSK_ALERT_REMINDER.REMINDER_DATE
                    )
                where
                    FSK_ALERT_REMINDER.ALERT_ID IS NULL 
        ;
    quit;

    %fcf_rcset(&sqlrc);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
        %do;
            %let _fcfmsg_=ERROR: FCF: IEM: Unable to get Alert Reminder updates to be appended to FSK_ALERT_REMINDER;

            %fcf_put_error;
            %goto MACRO_END;
        %end;

    /*==========================================================================* 
     * Step:            Table Loader                                            * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Alert Reminder Stage -                                  * 
     *                   iem.alert_reminder_stage                               * 
     * Target Table:    FSK_ALERT_REMINDER -                                    * 
     *                   &db_kc_libname..FSK_ALERT_REMINDER                     * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(iem.alert_reminder_stage);

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

    proc sql;
        create view work.temp_map as
            select
                ALERT_ID,
                ALT_REMINDER_DT as REMINDER_DATE,
                SOURCE_SYSTEM_CD as SEGMENT_ID,
                USER_LONG_ID,
                REMINDER_SENT_IND
            from &SOURCEDS
        ;
    quit;

    %let SOURCEDS = work.temp_map;

    proc append base = &db_kc_libname..FSK_ALERT_REMINDER 
        data = &SOURCEDS force;
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to append to FSK_RISK_ASSESSMENT for RAS updates;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

%MACRO_END:

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
        %fcf_execute_macro(fcf_error);
    %else
        %do;
            %fcf_execute_macro(fcf_success);
        %end;
%mend;
