/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_iem_fb_party.sas
| PRODUCT: SAS Financial Crimes Foundation
| SYSTEM:  Windows, Unix
| DESC:    ECM to FCF Party/Subject creation and synchronization
+------------------------------------------------------------------------------+
| USAGE:   %fcf_iem_fb_party;
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_iem_fb_party;

	/*------------------------------------------
	 * Set error code to 0 and message to blank
 	*------------------------------------------*/
	%let trans_rc = 0;
	%let _fcfmsg_=;

	/*==========================================================================* 
	 * Step:            Get All Recent Parties                                  *
	 * Transform:       Extract                                                 * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    PARTY_LIVE - ecm_db.PARTY_LIVE                          *
	 * Target Table:    All recent parties -                                    *
	 *                   work.all_recent_parties                                * 
 	*==========================================================================*/

	%let SOURCEDS = %nrquote(ecm_db.PARTY_LIVE);

	proc datasets lib = work nolist nowarn memtype = (data view);
		delete all_recent_parties;
	quit;

	proc sql;
		create view work.all_recent_parties as
			select
				PARTY_RK,
				VALID_FROM_DTTM
			from &SOURCEDS
				where date() - datepart(VALID_FROM_DTTM) <= 100 & PARTY_TYPE_CD = 'FCFSUB' & PARTY_CATEGORY_CD = 'PTY' & SOURCE_SYSTEM_CD = "&segKCschema"
		;
	quit;

	%let SOURCEDS = work.all_recent_parties;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to retrieve all FCF Parties from PARTY_LIVE;

		%fcf_put_error;
		%goto MACRO_END;
	%end;

	/*==========================================================================* 
	 * Step:            Get Risk Classifications                                *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   All recent parties -                                    *
	 *                   work.all_recent_parties                                * 
	 *                  PARTY_UDF_NUM_VALUE -                                   *
	 *                   ecm_db.PARTY_UDF_NUM_VALUE                             * 
	 * Target Table:    All Recent Parties With Risk                            *
	 *                  Classifications -                                       * 
	 *                   work.all_recent_w_rc                                   * 
	*==========================================================================*/

	proc datasets lib = work nolist nowarn memtype = (data view);
		delete all_recent_w_rc;
	quit;

	proc sql;
		create view work.all_recent_w_rc as
			select
				all_recent_parties.PARTY_RK,
				all_recent_parties.VALID_FROM_DTTM,
				PARTY_UDF_NUM_VALUE.UDF_VALUE as X_SASFCS_RSK_CLSSF
			from
				work.all_recent_parties left join 
				ecm_db.PARTY_UDF_NUM_VALUE
				on
				(
				all_recent_parties.PARTY_RK = PARTY_UDF_NUM_VALUE.PARTY_RK
				and all_recent_parties.VALID_FROM_DTTM = PARTY_UDF_NUM_VALUE.VALID_FROM_DTTM
					)
				where
					PARTY_UDF_NUM_VALUE.UDF_NM = 'X_SASFCS_RSK_CLSSF'
		;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to retrieve all Risk Classifications from PARTY_UDF_NUM_VALUE;

		%fcf_put_error;
		%goto MACRO_END;
	%end;

	/*==========================================================================* 
	 * Step:            Get Party Keys                                          *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   All Recent Parties With Risk                            *
	 *                  Classifications -                                       * 
	 *                   work.all_recent_w_rc                                   * 
	 *                  PARTY_UDF_NUM_VALUE -                                   *
	 *                   ecm_db.PARTY_UDF_NUM_VALUE                             * 
	 * Target Table:    All Recent Parties With Risk                            *
	 *                  Classifications And Keys -                              * 
	 *                   work.all_recent_w_rc_key                               * 
	 *==========================================================================*/

	proc datasets lib = work nolist nowarn memtype = (data view);
		delete all_recent_w_rc_key;
	quit;

	proc sql;
		create view work.all_recent_w_rc_key as
			select
				all_recent_w_rc.PARTY_RK,
				all_recent_w_rc.VALID_FROM_DTTM,
				all_recent_w_rc.X_SASFCS_RSK_CLSSF,
				PARTY_UDF_NUM_VALUE.UDF_VALUE as X_SASFCS_PTY_KEY
			from
				work.all_recent_w_rc left join 
				ecm_db.PARTY_UDF_NUM_VALUE
				on
				(
				all_recent_w_rc.PARTY_RK = PARTY_UDF_NUM_VALUE.PARTY_RK
				and all_recent_w_rc.VALID_FROM_DTTM = PARTY_UDF_NUM_VALUE.VALID_FROM_DTTM
					)
				where
					PARTY_UDF_NUM_VALUE.UDF_NM = 'X_SASFCS_PTY_KEY'
		;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to retrieve all Party Keys from PARTY_UDF_NUM_VALUE;

		%fcf_put_error;
		%goto MACRO_END;
	%end;

	/*==========================================================================* 
	 * Step:            Get Only New Party Updates                              *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   All Recent Parties With Risk                            *
	 *                  Classifications And Keys -                              * 
	 *                   work.all_recent_w_rc_key                               * 
	 *                  FSC_PARTY_DIM - &db_core_libname..FSC_PARTY_DIM         *
	 * Target Table:    New Party Updates -                                     *
	 *                   work.new_pty_updates                                   * 
	 *==========================================================================*/

	proc datasets lib = work nolist nowarn memtype = (data view);
		delete new_pty_updates;
	quit;

	proc sql;
		create view work.new_pty_updates as
			select
				all_recent_w_rc_key.X_SASFCS_PTY_KEY,
				all_recent_w_rc_key.X_SASFCS_RSK_CLSSF
			from
				work.all_recent_w_rc_key left join 
				&db_core_libname..FSC_PARTY_DIM
				on
				(
				all_recent_w_rc_key.X_SASFCS_PTY_KEY = FSC_PARTY_DIM.PARTY_KEY
				and all_recent_w_rc_key.X_SASFCS_RSK_CLSSF = FSC_PARTY_DIM.RISK_CLASSIFICATION
					)
				where
					FSC_PARTY_DIM.PARTY_KEY IS NULL 
		;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to filter new Party updates;

		%fcf_put_error;
		%goto MACRO_END;
	%end;

	/*==========================================================================* 
	 * Step:            Get Party Info                                          *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   New Party Updates -                                     *
	 *                   work.new_pty_updates                                   * 
	 *                  FSC_PARTY_DIM - &db_core_libname..FSC_PARTY_DIM         *
	 * Target Table:    New Party Updates With Info -                           *
	 *                   work.new_pty_updates_w_info                            * 
	 *==========================================================================*/

	proc datasets lib = work nolist nowarn memtype = (data view);
		delete new_pty_updates_w_info;
	quit;

	proc sql;
		create view work.new_pty_updates_w_info as
			select distinct
				new_pty_updates.X_SASFCS_PTY_KEY as PARTY_KEY,
				FSC_PARTY_DIM.SEGMENT_ID,
				FSC_PARTY_DIM.PARTY_NUMBER,
				FSC_PARTY_DIM.PARTY_TYPE_DESC,
				FSC_PARTY_DIM.PARTY_TAX_ID,
				FSC_PARTY_DIM.PARTY_TAX_ID_TYPE_CODE,
				FSC_PARTY_DIM.PARTY_IDENTIFICATION_ID,
				FSC_PARTY_DIM.PARTY_IDENTIFICATION_TYPE_DESC,
				FSC_PARTY_DIM.PARTY_ID_STATE_CODE,
				FSC_PARTY_DIM.PARTY_DATE_OF_BIRTH,
				FSC_PARTY_DIM.PARTY_FIRST_NAME,
				FSC_PARTY_DIM.PARTY_LAST_NAME,
				FSC_PARTY_DIM.PARTY_MIDDLE_NAME,
				FSC_PARTY_DIM.PARTY_NAME,
				FSC_PARTY_DIM.DOING_BUSINESS_AS_NAME,
				FSC_PARTY_DIM.PARTY_STATUS_DESC,
				FSC_PARTY_DIM.ULTIMATE_PARENT_NAME,
				FSC_PARTY_DIM.MATCH_CODE_ORGANIZATION,
				FSC_PARTY_DIM.MATCH_CODE_INDIVIDUAL,
				FSC_PARTY_DIM.MATCH_CODE_STREET_ADDRESS,
				FSC_PARTY_DIM.MATCH_CODE_MAILING_ADDRESS,
				FSC_PARTY_DIM.MATCH_CODE_STREET_ADDR_LINES,
				FSC_PARTY_DIM.MATCH_CODE_STREET_CITY,
				FSC_PARTY_DIM.MATCH_CODE_STREET_STATE,
				FSC_PARTY_DIM.MATCH_CODE_STREET_COUNTRY,
				FSC_PARTY_DIM.MATCH_CODE_MAILING_ADDR_LINES,
				FSC_PARTY_DIM.MATCH_CODE_MAILING_CITY,
				FSC_PARTY_DIM.MATCH_CODE_MAILING_STATE,
				FSC_PARTY_DIM.MATCH_CODE_MAILING_COUNTRY,
				FSC_PARTY_DIM.STREET_ADDRESS_1,
				FSC_PARTY_DIM.STREET_ADDRESS_2,
				FSC_PARTY_DIM.STREET_CITY_NAME,
				FSC_PARTY_DIM.STREET_STATE_CODE,
				FSC_PARTY_DIM.STREET_STATE_NAME,
				FSC_PARTY_DIM.STREET_POSTAL_CODE,
				FSC_PARTY_DIM.STREET_COUNTRY_CODE,
				FSC_PARTY_DIM.STREET_COUNTRY_NAME,
				FSC_PARTY_DIM.MAILING_ADDRESS_1,
				FSC_PARTY_DIM.MAILING_ADDRESS_2,
				FSC_PARTY_DIM.MAILING_CITY_NAME,
				FSC_PARTY_DIM.MAILING_STATE_CODE,
				FSC_PARTY_DIM.MAILING_STATE_NAME,
				FSC_PARTY_DIM.MAILING_POSTAL_CODE,
				FSC_PARTY_DIM.MAILING_COUNTRY_CODE,
				FSC_PARTY_DIM.MAILING_COUNTRY_NAME,
				FSC_PARTY_DIM.RESIDENCE_COUNTRY_CODE,
				FSC_PARTY_DIM.RESIDENCE_COUNTRY_NAME,
				FSC_PARTY_DIM.CITIZENSHIP_COUNTRY_CODE,
				FSC_PARTY_DIM.CITIZENSHIP_COUNTRY_NAME,
				FSC_PARTY_DIM.ORG_COUNTRY_OF_BUSINESS_CODE,
				FSC_PARTY_DIM.ORG_COUNTRY_OF_BUSINESS_NAME,
				FSC_PARTY_DIM.EMPLOYEE_IND,
				FSC_PARTY_DIM.EMPLOYEE_NUMBER,
				FSC_PARTY_DIM.EMPLOYER_NAME,
				FSC_PARTY_DIM.EMPLOYER_PHONE_NUMBER,
				FSC_PARTY_DIM.EMAIL_ADDRESS,
				FSC_PARTY_DIM.OCCUPATION_DESC,
				FSC_PARTY_DIM.INDUSTRY_CODE,
				FSC_PARTY_DIM.INDUSTRY_DESC,
				FSC_PARTY_DIM.PHONE_NUMBER_1,
				FSC_PARTY_DIM.PHONE_NUMBER_2,
				FSC_PARTY_DIM.PHONE_NUMBER_3,
				FSC_PARTY_DIM.ANNUAL_INCOME_AMOUNT,
				FSC_PARTY_DIM.NET_WORTH_AMOUNT,
				FSC_PARTY_DIM.MARITAL_STATUS_DESC,
				FSC_PARTY_DIM.LAST_CONTACT_DATE,
				FSC_PARTY_DIM.POLITICALLY_EXPOSED_PERSON_IND,
				FSC_PARTY_DIM.NON_PROFIT_ORG_IND,
				FSC_PARTY_DIM.CUSTOMER_SINCE_DATE,
				FSC_PARTY_DIM.LAST_SUSP_ACTV_RPT_DATE,
				FSC_PARTY_DIM.LAST_CASH_TRANS_RPT_DATE,
				FSC_PARTY_DIM.LAST_RISK_ASSESSMENT_DATE,
				new_pty_updates.X_SASFCS_RSK_CLSSF as RISK_CLASSIFICATION,
				FSC_PARTY_DIM.CHANGE_BEGIN_DATE,
				FSC_PARTY_DIM.CHANGE_END_DATE,
				FSC_PARTY_DIM.CHANGE_CURRENT_IND,
				FSC_PARTY_DIM.EXTERNAL_PARTY_IND,
				FSC_PARTY_DIM.LEGAL_ENTITY_TYPE,
				FSC_PARTY_DIM.MONEY_ORDERS_IND,
				FSC_PARTY_DIM.TRAVELERS_CHEQUES_IND,
				FSC_PARTY_DIM.PREPAID_CARDS_IND,
				FSC_PARTY_DIM.MSB_IND,
				FSC_PARTY_DIM.MONEY_TRANSMITTER_IND,
				FSC_PARTY_DIM.CURRENCY_EXCHANGE_IND,
				FSC_PARTY_DIM.CHECK_CASHER_IND,
				FSC_PARTY_DIM.INTERNET_GAMBLING_IND,
				FSC_PARTY_DIM.TRUST_ACCOUNT_IND,
				FSC_PARTY_DIM.FOREIGN_CONSULATE_EMBASSY_IND,
				FSC_PARTY_DIM.ISSUES_BEARER_SHARES_IND,
				FSC_PARTY_DIM.NEGATIVE_NEWS_IND
			from
				work.new_pty_updates left join 
				&db_core_libname..FSC_PARTY_DIM
				on
				(
				new_pty_updates.X_SASFCS_PTY_KEY = FSC_PARTY_DIM.PARTY_KEY
					)
		;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to retrieve Party info from FSC_PARTY_DIM;

		%fcf_put_error;
		%goto MACRO_END;
	%end;

	/*==========================================================================* 
	 * Step:            Load to staging                                         *
	 * Transform:       Table Loader (Version 2.1)                              * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    New Party Updates With Info -                           *
	 *                   work.new_pty_updates_w_info                            * 
	 * Target Table:    New Party Updates -                                     *
	 *                   iem.new_party_updates                                  * 
	 *==========================================================================*/

	%let SOURCEDS = %nrquote(work.new_pty_updates_w_info);

	data _null_;
		call symput('current_dt',put(datetime(),datetime22.));
	run;

	proc datasets lib = iem nolist nowarn memtype = (data view);
		delete new_party_updates;
	quit;

	proc sql;
		create table iem.new_party_updates as
			select
				PARTY_KEY,
				SEGMENT_ID,
				PARTY_NUMBER,
				PARTY_TYPE_DESC,
				PARTY_TAX_ID,
				PARTY_TAX_ID_TYPE_CODE,
				PARTY_IDENTIFICATION_ID,
				PARTY_IDENTIFICATION_TYPE_DESC,
				PARTY_ID_STATE_CODE,
				PARTY_DATE_OF_BIRTH,
				PARTY_FIRST_NAME,
				PARTY_LAST_NAME,
				PARTY_MIDDLE_NAME,
				PARTY_NAME,
				DOING_BUSINESS_AS_NAME,
				PARTY_STATUS_DESC,
				ULTIMATE_PARENT_NAME,
				MATCH_CODE_ORGANIZATION,
				MATCH_CODE_INDIVIDUAL,
				MATCH_CODE_STREET_ADDRESS,
				MATCH_CODE_MAILING_ADDRESS,
				MATCH_CODE_STREET_ADDR_LINES,
				MATCH_CODE_STREET_CITY,
				MATCH_CODE_STREET_STATE,
				MATCH_CODE_STREET_COUNTRY,
				MATCH_CODE_MAILING_ADDR_LINES,
				MATCH_CODE_MAILING_CITY,
				MATCH_CODE_MAILING_STATE,
				MATCH_CODE_MAILING_COUNTRY,
				STREET_ADDRESS_1,
				STREET_ADDRESS_2,
				STREET_CITY_NAME,
				STREET_STATE_CODE,
				STREET_STATE_NAME,
				STREET_POSTAL_CODE,
				STREET_COUNTRY_CODE,
				STREET_COUNTRY_NAME,
				MAILING_ADDRESS_1,
				MAILING_ADDRESS_2,
				MAILING_CITY_NAME,
				MAILING_STATE_CODE,
				MAILING_STATE_NAME,
				MAILING_POSTAL_CODE,
				MAILING_COUNTRY_CODE,
				MAILING_COUNTRY_NAME,
				RESIDENCE_COUNTRY_CODE,
				RESIDENCE_COUNTRY_NAME,
				CITIZENSHIP_COUNTRY_CODE,
				CITIZENSHIP_COUNTRY_NAME,
				ORG_COUNTRY_OF_BUSINESS_CODE,
				ORG_COUNTRY_OF_BUSINESS_NAME,
				EMPLOYEE_IND,
				EMPLOYEE_NUMBER,
				EMPLOYER_NAME,
				EMPLOYER_PHONE_NUMBER,
				EMAIL_ADDRESS,
				OCCUPATION_DESC,
				INDUSTRY_CODE,
				INDUSTRY_DESC,
				PHONE_NUMBER_1,
				PHONE_NUMBER_2,
				PHONE_NUMBER_3,
				ANNUAL_INCOME_AMOUNT,
				NET_WORTH_AMOUNT,
				MARITAL_STATUS_DESC,
				LAST_CONTACT_DATE,
				POLITICALLY_EXPOSED_PERSON_IND,
				NON_PROFIT_ORG_IND,
				CUSTOMER_SINCE_DATE,
				LAST_SUSP_ACTV_RPT_DATE,
				LAST_CASH_TRANS_RPT_DATE,
				LAST_RISK_ASSESSMENT_DATE,
				RISK_CLASSIFICATION,
				CHANGE_BEGIN_DATE,
				("&current_dt"dt) as CHANGE_END_DATE,
				CHANGE_CURRENT_IND,
				EXTERNAL_PARTY_IND,
				LEGAL_ENTITY_TYPE,
				MONEY_ORDERS_IND,
				TRAVELERS_CHEQUES_IND,
				PREPAID_CARDS_IND,
				MSB_IND,
				MONEY_TRANSMITTER_IND,
				CURRENCY_EXCHANGE_IND,
				CHECK_CASHER_IND,
				INTERNET_GAMBLING_IND,
				TRUST_ACCOUNT_IND,
				FOREIGN_CONSULATE_EMBASSY_IND,
				ISSUES_BEARER_SHARES_IND,
				NEGATIVE_NEWS_IND,
				. as NEW_PARTY_KEY,
				("&current_dt"dt+1) as NEW_CHANGE_BEGIN_DATE,
				CHANGE_END_DATE as NEW_CHANGE_END_DATE
			from &SOURCEDS
		;
	quit;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to create IEM.NEW_PARTY_UPDATES;

		%fcf_put_error;
		%goto MACRO_END;
	%end;

	%let obs_count = %fcf_get_observation_count(table=iem.new_party_updates, SASDATASET=Y);
	%let numevents = &obs_count;

	%fcf_get_sequence_ids(numIDs=&numevents);

	data iem.new_party_updates;
		set iem.new_party_updates;
		new_party_key = symgetn('seqId'||(kstrip(put(_n_,best32.))));
	run;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to create new Party keys for FSC_PARTY_DIM load;

		%fcf_put_error;
		%goto MACRO_END;
	%end;

	/*==========================================================================* 
	 * Step:            Update                                                  *
	 * Transform:       Update                                                  * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    New Party Updates -                                     *
	 *                   iem.new_party_updates                                  * 
	 * Target Table:    FSC_PARTY_DIM -                                         *
	 *                   &db_core_libname..FSC_PARTY_DIM                        *
 	*==========================================================================*/

	%let SOURCEDS = %nrquote(iem.new_party_updates);

	proc sql;
		update &db_core_libname..FSC_PARTY_DIM
			set 
				CHANGE_END_DATE= (
			select
				new_party_updates.CHANGE_END_DATE
			from
				iem.new_party_updates as new_party_updates
			where
				FSC_PARTY_DIM.PARTY_KEY = new_party_updates.PARTY_KEY
				and FSC_PARTY_DIM.SEGMENT_ID = new_party_updates.SEGMENT_ID
				),
				CHANGE_CURRENT_IND= 'N'
			where
				PARTY_KEY IN (
			select
				new_party_updates.PARTY_KEY
			from
				iem.new_party_updates as new_party_updates
				)
				and SEGMENT_ID IN (
			select
				new_party_updates.SEGMENT_ID
			from
				iem.new_party_updates as new_party_updates
				)
				and CHANGE_CURRENT_IND = 'Y'
		;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to update FSC_PARTY_DIM;

		%fcf_put_error;
		%goto MACRO_END;
	%end;

	/*==========================================================================* 
	 * Step:            Table Loader                                            *
	 * Transform:       Table Loader (Version 2.1)                              * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    New Party Updates -                                     *
	 *                   iem.new_party_updates                                  * 
	 * Target Table:    FSC_PARTY_DIM -                                         *
	 *                   &db_core_libname..FSC_PARTY_DIM                        *
 	*==========================================================================*/

	%let SOURCEDS = %nrquote(iem.new_party_updates);

	proc datasets lib = work nolist nowarn memtype = (data view);
		delete temp_map;
	quit;

	proc sql;
		create view work.temp_map as
			select
				NEW_PARTY_KEY as PARTY_KEY,
				SEGMENT_ID,
				PARTY_NUMBER,
				PARTY_TYPE_DESC,
				PARTY_TAX_ID,
				PARTY_TAX_ID_TYPE_CODE,
				PARTY_IDENTIFICATION_ID,
				PARTY_IDENTIFICATION_TYPE_DESC,
				PARTY_ID_STATE_CODE,
				PARTY_DATE_OF_BIRTH,
				PARTY_FIRST_NAME,
				PARTY_LAST_NAME,
				PARTY_MIDDLE_NAME,
				PARTY_NAME,
				DOING_BUSINESS_AS_NAME,
				PARTY_STATUS_DESC,
				ULTIMATE_PARENT_NAME,
				MATCH_CODE_ORGANIZATION,
				MATCH_CODE_INDIVIDUAL,
				MATCH_CODE_STREET_ADDRESS,
				MATCH_CODE_MAILING_ADDRESS,
				MATCH_CODE_STREET_ADDR_LINES,
				MATCH_CODE_STREET_CITY,
				MATCH_CODE_STREET_STATE,
				MATCH_CODE_STREET_COUNTRY,
				MATCH_CODE_MAILING_ADDR_LINES,
				MATCH_CODE_MAILING_CITY,
				MATCH_CODE_MAILING_STATE,
				MATCH_CODE_MAILING_COUNTRY,
				STREET_ADDRESS_1,
				STREET_ADDRESS_2,
				STREET_CITY_NAME,
				STREET_STATE_CODE,
				STREET_STATE_NAME,
				STREET_POSTAL_CODE,
				STREET_COUNTRY_CODE,
				STREET_COUNTRY_NAME,
				MAILING_ADDRESS_1,
				MAILING_ADDRESS_2,
				MAILING_CITY_NAME,
				MAILING_STATE_CODE,
				MAILING_STATE_NAME,
				MAILING_POSTAL_CODE,
				MAILING_COUNTRY_CODE,
				MAILING_COUNTRY_NAME,
				RESIDENCE_COUNTRY_CODE,
				RESIDENCE_COUNTRY_NAME,
				CITIZENSHIP_COUNTRY_CODE,
				CITIZENSHIP_COUNTRY_NAME,
				ORG_COUNTRY_OF_BUSINESS_CODE,
				ORG_COUNTRY_OF_BUSINESS_NAME,
				EMPLOYEE_IND,
				EMPLOYEE_NUMBER,
				EMPLOYER_NAME,
				EMPLOYER_PHONE_NUMBER,
				EMAIL_ADDRESS,
				OCCUPATION_DESC,
				INDUSTRY_CODE,
				INDUSTRY_DESC,
				PHONE_NUMBER_1,
				PHONE_NUMBER_2,
				PHONE_NUMBER_3,
				ANNUAL_INCOME_AMOUNT,
				NET_WORTH_AMOUNT,
				MARITAL_STATUS_DESC,
				LAST_CONTACT_DATE,
				POLITICALLY_EXPOSED_PERSON_IND,
				NON_PROFIT_ORG_IND,
				CUSTOMER_SINCE_DATE,
				LAST_SUSP_ACTV_RPT_DATE,
				LAST_CASH_TRANS_RPT_DATE,
				LAST_RISK_ASSESSMENT_DATE,
				RISK_CLASSIFICATION,
				NEW_CHANGE_BEGIN_DATE as CHANGE_BEGIN_DATE,
				NEW_CHANGE_END_DATE as CHANGE_END_DATE,
				CHANGE_CURRENT_IND,
				EXTERNAL_PARTY_IND,
				LEGAL_ENTITY_TYPE,
				MONEY_ORDERS_IND,
				TRAVELERS_CHEQUES_IND,
				PREPAID_CARDS_IND,
				MSB_IND,
				MONEY_TRANSMITTER_IND,
				CURRENCY_EXCHANGE_IND,
				CHECK_CASHER_IND,
				INTERNET_GAMBLING_IND,
				TRUST_ACCOUNT_IND,
				FOREIGN_CONSULATE_EMBASSY_IND,
				ISSUES_BEARER_SHARES_IND,
				NEGATIVE_NEWS_IND
			from &SOURCEDS
		;
	quit;

	%let SOURCEDS = work.temp_map;

	proc append base = &db_core_libname..FSC_PARTY_DIM 
		data = &SOURCEDS force;
	run;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to load into FSC_PARTY_DIM;

		%fcf_put_error;
		%goto MACRO_END;
	%end;

	proc datasets lib = work nolist nowarn memtype = (data view);
		delete temp_map;
	quit;

	/*==========================================================================* 
	 * Step:            Join                                                    *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   New Party Updates -                                     *
	 *                   iem.new_party_updates                                  * 
	 *                  FSC_PARTY_ACCOUNT_BRIDGE -                              *
	 *                   &db_core_libname..FSC_PARTY_ACCOUNT_BRIDGE             * 
	 * Target Table:    Join - work.all_party_accounts                          *
	 *==========================================================================*/

	proc datasets lib = work nolist nowarn memtype = (data view);
		delete all_party_accounts;
	quit;

	proc sql;
		create table work.all_party_accounts as
			select
				FSC_PARTY_ACCOUNT_BRIDGE.ACCOUNT_KEY,
				FSC_PARTY_ACCOUNT_BRIDGE.PARTY_KEY,
				FSC_PARTY_ACCOUNT_BRIDGE.ROLE_KEY,
				FSC_PARTY_ACCOUNT_BRIDGE.CHANGE_BEGIN_DATE,
				FSC_PARTY_ACCOUNT_BRIDGE.SEGMENT_ID,
				FSC_PARTY_ACCOUNT_BRIDGE.ROLE_DESC,
				datetime() as CHANGE_END_DATE,
				FSC_PARTY_ACCOUNT_BRIDGE.CHANGE_CURRENT_IND,
				new_party_updates.NEW_PARTY_KEY,
				datetime()+1 as NEW_CHANGE_BEGIN_DATE,
				new_party_updates.NEW_CHANGE_END_DATE
			from
				iem.new_party_updates, 
				&db_core_libname..FSC_PARTY_ACCOUNT_BRIDGE
			where
				new_party_updates.PARTY_KEY = FSC_PARTY_ACCOUNT_BRIDGE.PARTY_KEY
				and new_party_updates.SEGMENT_ID = FSC_PARTY_ACCOUNT_BRIDGE.SEGMENT_ID
				and FSC_PARTY_ACCOUNT_BRIDGE.CHANGE_CURRENT_IND = 'Y'
		;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_PARTY;

		%fcf_put_error;
		%goto MACRO_END;
	%end;

	/*==========================================================================* 
	 * Step:            Update                                                  *
	 * Transform:       Update                                                  * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Join - work.all_party_accounts                          *
	 * Target Table:    FSC_PARTY_ACCOUNT_BRIDGE -                              *
	 *                   &db_core_libname..FSC_PARTY_ACCOUNT_BRIDGE             *
	 *==========================================================================*/

	%let SOURCEDS = %nrquote(work.all_party_accounts);

	proc sql;
		update &db_core_libname..FSC_PARTY_ACCOUNT_BRIDGE
			set 
				CHANGE_END_DATE= (
			select
				all_party_accounts.CHANGE_END_DATE
			from
				work.all_party_accounts as all_party_accounts
			where
				FSC_PARTY_ACCOUNT_BRIDGE.ACCOUNT_KEY = all_party_accounts.ACCOUNT_KEY
				and FSC_PARTY_ACCOUNT_BRIDGE.PARTY_KEY = all_party_accounts.PARTY_KEY
				and FSC_PARTY_ACCOUNT_BRIDGE.SEGMENT_ID = all_party_accounts.SEGMENT_ID
				),
				CHANGE_CURRENT_IND= 'N'
			where
				PARTY_KEY IN (
			select
				all_party_accounts.PARTY_KEY
			from
				work.all_party_accounts as all_party_accounts
				)
				and SEGMENT_ID IN (
			select
				all_party_accounts.SEGMENT_ID
			from
				work.all_party_accounts as all_party_accounts
				)
				and CHANGE_CURRENT_IND = 'Y'
		;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_PARTY;

		%fcf_put_error;
		%goto MACRO_END;
	%end;

	/*==========================================================================* 
	 * Step:            Table Loader                                            *
	 * Transform:       Table Loader (Version 2.1)                              * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Join - work.all_party_accounts                          *
	 * Target Table:    FSC_PARTY_ACCOUNT_BRIDGE -                              *
	 *                   &db_core_libname..FSC_PARTY_ACCOUNT_BRIDGE             * 
	 *==========================================================================*/

	%let SOURCEDS = %nrquote(work.all_party_accounts);

	proc datasets lib = work nolist nowarn memtype = (data view);
		delete temp_map;
	quit;

	data temp_map;
		if 0 then
			set &db_core_libname..FSC_PARTY_ACCOUNT_BRIDGE;
		set &SOURCEDS. (keep= NEW_PARTY_KEY ACCOUNT_KEY ROLE_KEY SEGMENT_ID ROLE_DESC NEW_CHANGE_BEGIN_DATE NEW_CHANGE_END_DATE);
		PARTY_KEY = NEW_PARTY_KEY;
		ACCOUNT_KEY = ACCOUNT_KEY;
		ROLE_KEY = ROLE_KEY;
		SEGMENT_ID = SEGMENT_ID;
		ROLE_DESC = ROLE_DESC;
		CHANGE_BEGIN_DATE = NEW_CHANGE_BEGIN_DATE;
		CHANGE_END_DATE = NEW_CHANGE_END_DATE;
		CHANGE_CURRENT_IND = 'Y';
		drop NEW_PARTY_KEY NEW_CHANGE_BEGIN_DATE NEW_CHANGE_END_DATE;
	run;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_PARTY;

		%fcf_put_error;
		%goto MACRO_END;
	%end;

	%let SOURCEDS = work.temp_map;

	proc append base = &db_core_libname..FSC_PARTY_ACCOUNT_BRIDGE 
		data = &SOURCEDS force;
	run;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_PARTY;

		%fcf_put_error;
		%goto MACRO_END;
	%end;

	proc datasets lib = work nolist nowarn memtype = (data view);
		delete temp_map;
	quit;

	/*==========================================================================* 
	 * Step:            Join                                                    *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   New Party Updates -                                     *
	 *                   iem.new_party_updates                                  * 
	 *                  FSC_HOUSEHOLD_PARTY_BRIDGE -                            *
	 *                   &db_core_libname..FSC_HOUSEHOLD_PARTY_BRIDGE           * 
	 * Target Table:    Join - work.all_household_parties                       *
	 *==========================================================================*/

	proc datasets lib = work nolist nowarn memtype = (data view);
		delete all_household_parties;
	quit;

	proc sql;
		create table work.all_household_parties as
			select
				FSC_HOUSEHOLD_PARTY_BRIDGE.PARTY_KEY,
				FSC_HOUSEHOLD_PARTY_BRIDGE.HOUSEHOLD_KEY,
				FSC_HOUSEHOLD_PARTY_BRIDGE.CHANGE_BEGIN_DATE,
				FSC_HOUSEHOLD_PARTY_BRIDGE.SEGMENT_ID,
				datetime() as CHANGE_END_DATE,
				FSC_HOUSEHOLD_PARTY_BRIDGE.CHANGE_CURRENT_IND,
				new_party_updates.NEW_PARTY_KEY,
				datetime()+1 as NEW_CHANGE_BEGIN_DATE,
				new_party_updates.NEW_CHANGE_END_DATE
			from
				iem.new_party_updates, 
				&db_core_libname..FSC_HOUSEHOLD_PARTY_BRIDGE
			where
				new_party_updates.PARTY_KEY = FSC_HOUSEHOLD_PARTY_BRIDGE.PARTY_KEY
				and new_party_updates.SEGMENT_ID = FSC_HOUSEHOLD_PARTY_BRIDGE.SEGMENT_ID
				and FSC_HOUSEHOLD_PARTY_BRIDGE.CHANGE_CURRENT_IND = 'Y'
		;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_PARTY;

		%fcf_put_error;
		%goto MACRO_END;
	%end;

	/*==========================================================================* 
	 * Step:            Update                                                  *
	 * Transform:       Update                                                  * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Join - work.all_household_parties                       *
	 * Target Table:    FSC_HOUSEHOLD_PARTY_BRIDGE -                            *
	 *                   &db_core_libname..FSC_HOUSEHOLD_PARTY_BRIDGE           * 
	 *==========================================================================*/

	%let SOURCEDS = %nrquote(work.all_household_parties);

	proc sql;
		update &db_core_libname..FSC_HOUSEHOLD_PARTY_BRIDGE
			set 
				CHANGE_END_DATE= (
			select
				all_household_parties.CHANGE_END_DATE
			from
				work.all_household_parties as all_household_parties
			where
				FSC_HOUSEHOLD_PARTY_BRIDGE.PARTY_KEY = all_household_parties.PARTY_KEY
				and FSC_HOUSEHOLD_PARTY_BRIDGE.HOUSEHOLD_KEY = all_household_parties.HOUSEHOLD_KEY
				and FSC_HOUSEHOLD_PARTY_BRIDGE.SEGMENT_ID = all_household_parties.SEGMENT_ID
				),
				CHANGE_CURRENT_IND= 'N'
			where
				PARTY_KEY IN (
			select
				all_household_parties.PARTY_KEY
			from
				work.all_household_parties as all_household_parties
				)
				and SEGMENT_ID IN (
			select
				all_household_parties.SEGMENT_ID
			from
				work.all_household_parties as all_household_parties
				)
				and CHANGE_CURRENT_IND = 'Y'
		;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_PARTY;

		%fcf_put_error;
		%goto MACRO_END;
	%end;

	/*==========================================================================* 
	 * Step:            Table Loader                                            *
	 * Transform:       Table Loader (Version 2.1)                              * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Join - work.all_household_parties                       *
	 * Target Table:    FSC_HOUSEHOLD_PARTY_BRIDGE -                            *
	 *                   &db_core_libname..FSC_HOUSEHOLD_PARTY_BRIDGE            * 
	 *==========================================================================*/

	%let SOURCEDS = %nrquote(work.all_household_parties);

	proc datasets lib = work nolist nowarn memtype = (data view);
		delete temp_map;
	quit;

	data temp_map;
		if 0 then
			set &db_core_libname..FSC_HOUSEHOLD_PARTY_BRIDGE;
		set &SOURCEDS. (keep= NEW_PARTY_KEY HOUSEHOLD_KEY SEGMENT_ID NEW_CHANGE_BEGIN_DATE NEW_CHANGE_END_DATE);
		PARTY_KEY = NEW_PARTY_KEY;
		HOUSEHOLD_KEY = HOUSEHOLD_KEY;
		SEGMENT_ID = SEGMENT_ID;
		CHANGE_BEGIN_DATE = NEW_CHANGE_BEGIN_DATE;
		CHANGE_END_DATE = NEW_CHANGE_END_DATE;
		CHANGE_CURRENT_IND = 'Y';
		drop NEW_PARTY_KEY NEW_CHANGE_BEGIN_DATE NEW_CHANGE_END_DATE;
	run;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then
		%do;
			%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_PARTY;

			%fcf_put_error;
			%goto MACRO_END;
		%end;

	%let SOURCEDS = work.temp_map;

	proc append base = &db_core_libname..FSC_HOUSEHOLD_PARTY_BRIDGE 
		data = &SOURCEDS force;
	run;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then
		%do;
			%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_PARTY;

			%fcf_put_error;
			%goto MACRO_END;
		%end;

	proc datasets lib = work nolist nowarn memtype = (data view);
		delete temp_map;
	quit;

	/*==========================================================================* 
	 * Step:            Table Loader                                            *
	 * Transform:       Table Loader (Version 2.1)                              * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    New Party Updates -                                     *
	 *                   iem.new_party_updates                                  * 
	 * Target Table:    PARTY_TYPE2NEW - iem.PARTY_TYPE2NEW                     *
 	*==========================================================================*/

	%let SOURCEDS = %nrquote(iem.new_party_updates);

	proc datasets lib = iem nolist nowarn memtype = (data view);
		delete PARTY_TYPE2NEW;
	quit;

	data iem.PARTY_TYPE2NEW;
		if 0 then
			set &db_core_libname..FSC_PARTY_DIM;
		set &SOURCEDS. (keep= SEGMENT_ID NEW_PARTY_KEY PARTY_NUMBER PARTY_TAX_ID PARTY_NAME RISK_CLASSIFICATION);
		PARTY_KEY = NEW_PARTY_KEY;
		PARTY_NUMBER = PARTY_NUMBER;
		PARTY_TAX_ID = PARTY_TAX_ID;
		PARTY_NAME = PARTY_NAME;
		RISK_CLASSIFICATION = RISK_CLASSIFICATION;
		drop NEW_PARTY_KEY;
	run;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_PARTY;

		%fcf_put_error;
		%goto MACRO_END;
	%end;

	proc datasets lib = iem nolist nowarn memtype = (data view);
		delete PARTY_TYPE1UPDATE;
	quit;

	data iem.PARTY_TYPE1UPDATE;
		if 0 then
			set &db_core_libname..FSC_PARTY_DIM;
		stop;
	run;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_PARTY;

		%fcf_put_error;
		%goto MACRO_END;
	%end;

	/* Call Party Update to return new AML key to ECM */
	%fcf_iem_update_party

	%MACRO_END:

	%if &trans_rc eq 3 or &trans_rc ge 5 %then
		%fcf_execute_macro(fcf_error);
	%else %do;
		%fcf_execute_macro(fcf_success);
	%end;
%mend;