/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_iem_load_dimension_rels.sas
| PRODUCT: SAS Financial Crimes Foundation
| SYSTEM:  Windows, Unix
| DESC:    FCF to ECM Subject creation and synchronization
+------------------------------------------------------------------------------+
| USAGE:   %fcf_iem_load_dimension_rels(ds_HhdPty=, ds_HhdAcc=, ds_PtyAcc=);
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_iem_load_dimension_rels(ds_HhdPty=, ds_HhdAcc=, ds_PtyAcc=);

    /*------------------------------------------
     * Set error code to 0 and message to blank
     *------------------------------------------*/

    %let trans_rc = 0;
    %let _fcfmsg_=;


	/*==========================================================================* 
	 * Step:            Extract                                                 *
	 * Transform:       Extract                                                 * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    PARTY_LIVE - ecm_db.PARTY_LIVE                          *
	 * Target Table:    ECM Household RKs - work.hhd_rk                         *
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(ecm_db.PARTY_LIVE); 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete hhd_rk;
	quit;

	proc sql;
	   create view work.hhd_rk as
	      select
	         PARTY_RK,
	         (SUBSTR(PARTY_ID,5)) as PARTY_ID
	   from &SOURCEDS
	      where PARTY_TYPE_CD = "FCFSUB"  & PARTY_CATEGORY_CD = "HHD"
	   ;
	quit;

	%let SOURCEDS = work.hhd_rk;

    %fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_DIMENSION_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Extract                                                 *
	 * Transform:       Extract                                                 * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    PARTY_LIVE - ecm_db.PARTY_LIVE                          *
	 * Target Table:    ECM Account RKs - work.acc_rk                           *
	 *==========================================================================*/ 

    %let SOURCEDS = %nrquote(ecm_db.PARTY_LIVE); 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete acc_rk;
	quit;

	proc sql;
	   create view work.acc_rk as
	      select
	         PARTY_RK,
	         (SUBSTR(PARTY_ID,5)) as PARTY_ID
	   from &SOURCEDS
	      where PARTY_TYPE_CD = "FCFSUB"  & PARTY_CATEGORY_CD = "ACC"
	   ;
	quit;

	%let SOURCEDS = work.acc_rk;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_DIMENSION_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Extract                                                 *
	 * Transform:       Extract                                                 * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    PARTY_LIVE - ecm_db.PARTY_LIVE                          *
	 * Target Table:    ECM Party RKs - work.pty_rk                             *
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(ecm_db.PARTY_LIVE); 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete pty_rk;
	quit;

	proc sql;
	   create view work.pty_rk as
	      select
	         PARTY_RK,
	         (SUBSTR(PARTY_ID,5)) as PARTY_ID
	   from &SOURCEDS
	      where PARTY_TYPE_CD = "FCFSUB"  & PARTY_CATEGORY_CD = "PTY"
	   ;
	quit;

	%let SOURCEDS = work.pty_rk;

	%global etls_sql_pushDown;
	%let etls_sql_pushDown = &sys_sql_ip_all;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_DIMENSION_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Get HHD-PTY relationships                               *
	 * Transform:       Extract                                                 * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Household Party input ds -                              *
	 *                   &ds_HhdPty                                             * 
	 * Target Table:    New Household Parties - work.hhd_pty                    *
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(&ds_HhdPty); 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete hhd_pty;
	quit;

	proc sql;
	   create view work.hhd_pty as
	      select
	         party_key   
	            label = 'party_key',
	         household_key   
	            label = 'household_key',
	         ("HHD2PTY") as MEMBER_TYPE_CD
	   from &SOURCEDS
	      where CHANGE_CURRENT_IND = "Y" /* & SEGMENT_ID = "&segKCSchema"*/
	   ;
	quit;

	%let SOURCEDS = work.hhd_pty;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_DIMENSION_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Get HHD and ACC FCF natural keys                        *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   New Household Parties - work.hhd_pty                    *
	 *                  FSC_HOUSEHOLD_DIM -                                     *
	 *                   &db_core_libname..FSC_HOUSEHOLD_DIM                    * 
	 *                  FSC_PARTY_DIM - &db_core_libname..FSC_PARTY_DIM         *
	 * Target Table:    New HHD PTY with FCF keys -                             *
	 *                   work.hhd_pty_w_fcf                                     * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete hhd_pty_w_fcf;
	quit;

	proc sql;
	   create view work.hhd_pty_w_fcf as
	   select
	      FSC_HOUSEHOLD_DIM.HOUSEHOLD_NUMBER,
	      FSC_PARTY_DIM.PARTY_NUMBER,
	      hhd_pty.MEMBER_TYPE_CD
	   from
	      work.hhd_pty, 
	      &db_core_libname..FSC_HOUSEHOLD_DIM, 
	      &db_core_libname..FSC_PARTY_DIM
	   where
	      hhd_pty.household_key = FSC_HOUSEHOLD_DIM.HOUSEHOLD_KEY
	      and hhd_pty.party_key = FSC_PARTY_DIM.PARTY_KEY
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_DIMENSION_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Get HHD and PTY ECM keys                                *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   New HHD PTY with FCF keys -                             *
	 *                   work.hhd_pty_w_fcf                                     * 
	 *                  ECM Household RKs - work.hhd_rk                         *
	 *                  ECM Party RKs - work.pty_rk                             *
	 * Target Table:    New HHD PTY with FCF and ECM keys -                     *
	 *                   work.hhd_pty_w_fcf_ecm                                 * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete hhd_pty_w_fcf_ecm;
	quit;

	proc sql;
	   create view work.hhd_pty_w_fcf_ecm as
	   select
	      hhd_rk.PARTY_RK,
	      pty_rk.PARTY_RK as MEMBER_PARTY_RK,
	      hhd_pty_w_fcf.MEMBER_TYPE_CD
	   from
	      work.hhd_pty_w_fcf, 
	      work.hhd_rk, 
	      work.pty_rk
	   where
	      hhd_pty_w_fcf.HOUSEHOLD_NUMBER = hhd_rk.PARTY_ID
	      and hhd_pty_w_fcf.PARTY_NUMBER = pty_rk.PARTY_ID
	   ;
	quit;

    %fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_DIMENSION_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Get HHD-PTY relationships                               *
	 * Transform:       Extract                                                 * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Household Account input ds -                            *
	 *                   &ds_HhdAcc                                             * 
	 * Target Table:    New Household Accounts - work.hhd_acc                   *
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(&ds_HhdAcc); 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete hhd_acc;
	quit;

	proc sql;
	   create view work.hhd_acc as
	      select
	         household_key   
	            label = 'household_key',
	         account_key   
	            label = 'account_key',
	         ("HHD2ACC") as MEMBER_TYPE_CD
	   from &SOURCEDS
	      where CHANGE_CURRENT_IND = "Y" /*& SEGMENT_ID = "&segKCSchema"*/
	   ;
	quit;

    %fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_DIMENSION_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	%let SOURCEDS = work.hhd_acc;

	/*==========================================================================* 
	 * Step:            Get HHD and PTY FCF natural keys                        *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   New Household Accounts - work.hhd_acc                   *
	 *                  FSC_HOUSEHOLD_DIM -                                     *
	 *                   &db_core_libname..FSC_HOUSEHOLD_DIM                    * 
	 *                  FSC_ACCOUNT_DIM -                                       *
	 *                   &db_core_libname..FSC_ACCOUNT_DIM                      * 
	 * Target Table:    New HHD ACC with FCF keys -                             *
	 *                   work.hhd_acc_w_fcf                                     * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete hhd_acc_w_fcf;
	quit;

	proc sql;
	   create view work.hhd_acc_w_fcf as
	   select
	      FSC_HOUSEHOLD_DIM.HOUSEHOLD_NUMBER,
	      FSC_ACCOUNT_DIM.ACCOUNT_NUMBER,
	      hhd_acc.MEMBER_TYPE_CD
	   from
	      work.hhd_acc, 
	      &db_core_libname..FSC_HOUSEHOLD_DIM, 
	      &db_core_libname..FSC_ACCOUNT_DIM
	   where
	      hhd_acc.household_key = FSC_HOUSEHOLD_DIM.HOUSEHOLD_KEY
	      and hhd_acc.account_key = FSC_ACCOUNT_DIM.ACCOUNT_KEY
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_DIMENSION_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Get HHD and ACC ECM keys                                *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   New HHD ACC with FCF keys -                             *
	 *                   work.hhd_acc_w_fcf                                     * 
	 *                  ECM Household RKs - work.hhd_rk                         *
	 *                  ECM Account RKs - work.acc_rk                           *
	 * Target Table:    New HHD ACC with FCF and ECM keys -                     *
	 *                   work.hhd_acc_w_fcf_ecm                                 * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete hhd_acc_w_fcf_ecm;
	quit;

	proc sql;
	   create view work.hhd_acc_w_fcf_ecm as
	   select
	      hhd_rk.PARTY_RK,
	      acc_rk.PARTY_RK as MEMBER_PARTY_RK,
	      hhd_acc_w_fcf.MEMBER_TYPE_CD
	   from
	      work.hhd_acc_w_fcf, 
	      work.hhd_rk, 
	      work.acc_rk
	   where
	      hhd_acc_w_fcf.HOUSEHOLD_NUMBER = hhd_rk.PARTY_ID
	      and hhd_acc_w_fcf.ACCOUNT_NUMBER = acc_rk.PARTY_ID
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_DIMENSION_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Get Primary PTY-ACC relationships                       *
	 * Transform:       Extract                                                 * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Party Account input ds -                                *
	 *                   &ds_PtyAcc                                             * 
	 * Target Table:    New Primary Party Accounts -                            *
	 *                   work.pty_acc_pri                                       * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(&ds_PtyAcc); 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete pty_acc_pri;
	quit;

	proc sql;
	   create view work.pty_acc_pri as
	      select
	         account_key   
	            label = 'account_key',
	         party_key   
	            label = 'party_key',
	         ("PRI2ACC") as MEMBER_TYPE_CD 
	   from &SOURCEDS
	      where CHANGE_CURRENT_IND = "Y" /*& SEGMENT_ID = "&segKCSchema"  */& ROLE_KEY = 1
	   ;
	quit;

	%let SOURCEDS = work.pty_acc_pri;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_DIMENSION_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Get PTY and ACC FCF natural keys                        *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   FSC_PARTY_DIM - &db_core_libname..FSC_PARTY_DIM         *
	 *                  FSC_ACCOUNT_DIM -                                       *
	 *                   &db_core_libname..FSC_ACCOUNT_DIM                      * 
	 *                  New Primary Party Accounts -                            *
	 *                   work.pty_acc_pri                                       * 
	 * Target Table:    New Primary PTY ACC with FCF keys -                     *
	 *                   work.pri_pty_acc_w_fcf                                 * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete pri_pty_acc_w_fcf;
	quit;

	proc sql;
	   create view work.pri_pty_acc_w_fcf as
	   select
	      FSC_ACCOUNT_DIM.ACCOUNT_NUMBER,
	      FSC_PARTY_DIM.PARTY_NUMBER,
	      pty_acc_pri.MEMBER_TYPE_CD
	   from
	      work.pty_acc_pri left join 
	      &db_core_libname..FSC_PARTY_DIM
	         on
	         (
	            pty_acc_pri.party_key = FSC_PARTY_DIM.PARTY_KEY
	         ) left join 
	      &db_core_libname..FSC_ACCOUNT_DIM
	         on
	         (
	            pty_acc_pri.account_key = FSC_ACCOUNT_DIM.ACCOUNT_KEY
	         )
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_DIMENSION_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Get PTY and ACC ECM keys                                *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   New Primary PTY ACC with FCF keys -                     *
	 *                   work.pri_pty_acc_w_fcf                                 * 
	 *                  ECM Party RKs - work.pty_rk                             *
	 *                  ECM Account RKs - work.acc_rk                           *
	 * Target Table:    New Primary PTY ACC with FCF and ECM                    *
	 *                   keys - work.pri_pty_acc_w_fcf_ecm                      * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete pri_pty_acc_w_fcf_ecm;
	quit;

	proc sql;
	   create view work.pri_pty_acc_w_fcf_ecm as
	   select
	      pty_rk.PARTY_RK,
	      acc_rk.PARTY_RK as MEMBER_PARTY_RK,
	      pri_pty_acc_w_fcf.MEMBER_TYPE_CD
	   from
	      work.pri_pty_acc_w_fcf, 
	      work.pty_rk, 
	      work.acc_rk
	   where
	      pri_pty_acc_w_fcf.PARTY_NUMBER = pty_rk.PARTY_ID
	      and pri_pty_acc_w_fcf.ACCOUNT_NUMBER = acc_rk.PARTY_ID
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_DIMENSION_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Get PTY-ACC relationships                               *
	 * Transform:       Extract                                                 * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Party Account input ds -                                *
	 *                   &ds_PtyAcc                                             * 
	 * Target Table:    New non-primary Party Accounts -                        *
	 *                   work.pty_acc                                           * 
	 *==========================================================================*/ 

    %let SOURCEDS = %nrquote(&ds_PtyAcc); 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete pty_acc;
	quit;

	proc sql;
	   create view work.pty_acc as
	      select
	         ACCOUNT_KEY,
	         PARTY_KEY,
	         ("PTY2ACC") as MEMBER_TYPE_CD 
	   from &SOURCEDS
	      where CHANGE_CURRENT_IND = "Y" /*& SEGMENT_ID = "&segKCSchema" */ & ROLE_KEY ^= 1
	   ;
	quit;

	%let SOURCEDS = work.pty_acc;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_DIMENSION_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Get PTY and ACC FCF natural keys                        *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   FSC_PARTY_DIM - &db_core_libname..FSC_PARTY_DIM         *
	 *                  FSC_ACCOUNT_DIM -                                       *
	 *                   &db_core_libname..FSC_ACCOUNT_DIM                      * 
	 *                  New non-primary Party Accounts -                        *
	 *                   work.pty_acc                                           * 
	 * Target Table:    New non-primary PTY ACC with FCF keys                   *
	 *                   - work.pty_acc_w_fcf                                   * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete pty_acc_w_fcf;
	quit;

	proc sql;
	   create view work.pty_acc_w_fcf as
	   select
	      FSC_PARTY_DIM.PARTY_NUMBER,
	      FSC_ACCOUNT_DIM.ACCOUNT_NUMBER,
	      pty_acc.MEMBER_TYPE_CD
	   from
	      work.pty_acc left join 
	      &db_core_libname..FSC_PARTY_DIM
	         on
	         (
	            pty_acc.PARTY_KEY = FSC_PARTY_DIM.PARTY_KEY
	         ) left join 
	      &db_core_libname..FSC_ACCOUNT_DIM
	         on
	         (
	            pty_acc.ACCOUNT_KEY = FSC_ACCOUNT_DIM.ACCOUNT_KEY
	         )
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_DIMENSION_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Get PTY and ACC ECM keys                                *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   New non-primary PTY ACC with FCF keys                   *
	 *                   - work.pty_acc_w_fcf                                   * 
	 *                  ECM Party RKs - work.pty_rk                             *
	 *                  ECM Account RKs - work.acc_rk                           *
	 * Target Table:    New PTY ACC with FCF and ECM keys -                     *
	 *                   work.pty_acc_w_fcf_ecm                                 * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete pty_acc_w_fcf_ecm;
	quit;

	proc sql;
	   create view work.pty_acc_w_fcf_ecm as
	   select
	      pty_rk.PARTY_RK,
	      acc_rk.PARTY_RK as MEMBER_PARTY_RK,
	      pty_acc_w_fcf.MEMBER_TYPE_CD
	   from
	      work.pty_acc_w_fcf, 
	      work.pty_rk, 
	      work.acc_rk
	   where
	      pty_acc_w_fcf.PARTY_NUMBER = pty_rk.PARTY_ID
	      and pty_acc_w_fcf.ACCOUNT_NUMBER = acc_rk.PARTY_ID
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_DIMENSION_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Append                                                  *
	 * Transform:       Append                                                  * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   New HHD PTY with FCF and ECM keys -                     *
	 *                   work.hhd_pty_w_fcf_ecm                                 * 
	 *                  New HHD ACC with FCF and ECM keys -                     *
	 *                   work.hhd_acc_w_fcf_ecm                                 * 
	 *                  New Primary PTY ACC with FCF and ECM                    *
	 *                   keys - work.pri_pty_acc_w_fcf_ecm                      * 
	 *                  New PTY ACC with FCF and ECM keys -                     *
	 *                   work.pty_acc_w_fcf_ecm                                 * 
	 * Target Table:    All Party to Incident relationships -                   *
	 *                   work.all_pty_x_pty                                     * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete all_pty_x_pty;
	quit;

	data work.all_pty_x_pty / view=work.all_pty_x_pty;
	   set work.hhd_pty_w_fcf_ecm
	       work.hhd_acc_w_fcf_ecm
	       work.pri_pty_acc_w_fcf_ecm
	       work.pty_acc_w_fcf_ecm;
	   keep PARTY_RK MEMBER_PARTY_RK MEMBER_TYPE_CD; 
	run;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_DIMENSION_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Table Loader                                            *
	 * Transform:       Table Loader (Version 2.1)                              * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    All Party to Incident relationships -                   *
	 *                   work.all_pty_x_pty                                     * 
	 * Target Table:    PARTY_X_PARTY - ecm_db.PARTY_X_PARTY                    *
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.all_pty_x_pty); 

    proc datasets lib = iem nolist nowarn memtype = (data view);
        delete new_pty_x_pty;
    quit;

	data iem.new_pty_x_pty;
		if 0 then set ecm_db.party_x_party;
		set &SOURCEDS. (keep= PARTY_RK MEMBER_PARTY_RK MEMBER_TYPE_CD);
		PARTY_RK = PARTY_RK;
		MEMBER_PARTY_RK = MEMBER_PARTY_RK;
		MEMBER_TYPE_CD = MEMBER_TYPE_CD;
	run;

	%let SOURCEDS = iem.new_pty_x_pty;

	proc append base = ecm_db.PARTY_X_PARTY 
	  data = &SOURCEDS force; 
	run; 

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_DIMENSION_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Table Loader                                            *
	 * Transform:       Table Loader (Version 2.1)                              * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    All Party to Incident relationships -                   *
	 *                   work.all_pty_x_pty                                     * 
	 * Target Table:    ECM_EVENT - ecm_db.ECM_EVENT                            *
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.all_pty_x_pty); 

	data _null_;
    	call symput('current_dt',put(datetime(),datetime22.));
	run;

    proc datasets lib = iem nolist nowarn memtype = (data view);
        delete new_events;
    quit;

	data iem.new_events;
		if 0 then set ecm_db.ecm_event;
		set &SOURCEDS. (keep= PARTY_RK MEMBER_PARTY_RK);
		BUSINESS_OBJECT_NM = "PARTY";
		BUSINESS_OBJECT_RK = PARTY_RK;
		EVENT_TYPE_CD = "ADDPAR";
		EVENT_DESC = "event.etl.load.txt";
		CREATE_USER_ID = "&ecm_user";
		CREATE_DTTM = "&current_dt"dt;
		LINKED_OBJECT_NM = "PARTY";
		LINKED_OBJECT_RK = MEMBER_PARTY_RK;
		drop PARTY_RK MEMBER_PARTY_RK;
	run;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_DIMENSION_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	%let SOURCEDS = iem.new_events;
   
	%fcf_get_ecm_sequence_ids(in=iem.new_events,out=iem.new_events,type=event,block=&iem_performance_load);

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_DIMENSION_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	proc append base = ecm_db.ECM_EVENT
		data = &SOURCEDS force; 
	run; 

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_DIMENSION_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

%MACRO_END:

%if &trans_rc eq 3 or &trans_rc ge 5 %then
    %fcf_execute_macro(fcf_error);
%else
    %do;
        %fcf_execute_macro(fcf_success);
    %end;

%mend;
