/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_iem_load_risk_rels.sas
| PRODUCT: SAS Financial Crimes Foundation
| SYSTEM:  Windows, Unix
| DESC:    FCF to ECM Incident creation and synchronization
+------------------------------------------------------------------------------+
| USAGE:   %fcf_iem_load_risk_rels(ds=);
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_iem_load_risk_rels(ds=);

    /*------------------------------------------
     * Set error code to 0 and message to blank
     *------------------------------------------*/

    %let trans_rc = 0;
    %let _fcfmsg_=;

	/*==========================================================================* 
	 * Step:            Get PTY Subject keys                                    *
	 * Transform:       Extract                                                 * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    PARTY_LIVE - ecm_db.PARTY_LIVE                          *
	 * Target Table:    PTY Subject keys - work.pty_rk                          *
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(ecm_db.PARTY_LIVE); 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete pty_rk;
	quit;

	proc sql;
	   create view work.pty_rk as
	      select
	         PARTY_RK,
	         PARTY_ID
	   from &SOURCEDS
	      where PARTY_TYPE_CD = "FCFSUB"  & PARTY_CATEGORY_CD = "PTY"
	   ;
	quit;

	%let SOURCEDS = work.pty_rk;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_RISK_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Get Incident keys                                       *
	 * Transform:       Extract                                                 * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    INCIDENT_LIVE - ecm_db.INCIDENT_LIVE                    *
	 * Target Table:    Incident keys - work.ras_rk                             *
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(ecm_db.INCIDENT_LIVE); 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete ras_rk;
	quit;

	proc sql;
	   create view work.ras_rk as
	      select
	         INCIDENT_RK,
	         INCIDENT_ID
	   from &SOURCEDS
	      where INCIDENT_TYPE_CD = "AMLRISK"
	   ;
	quit;

	%let SOURCEDS = work.ras_rk;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_RISK_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Get Risk Assessments                                    *
	 * Transform:       Extract                                                 * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    FSK_RISK_ASSESSMENT -                                   *
	 *                   db_kc.FSK_RISK_ASSESSMENT                              * 
	 * Target Table:    Risk Assessments - work.ras                             *
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(db_kc.FSK_RISK_ASSESSMENT); 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete ras;
	quit;

	proc sql;
	   create view work.ras as
	      select
	         (CATX('-','RAS',compress(put(RISK_ASSESSMENT_ID,32.)))) as risk_assessment_id,
	         (CATX('-','PTY',PARTY_NUMBER)) as party_number
	   from &SOURCEDS
	      where SEGMENT_ID = "&segKCSchema"
	   ;
	quit;

	%let SOURCEDS = work.ras;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_RISK_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Get ECM keys for Risk Assessments                       *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   Risk Assessments - work.ras                             *
	 *                  PTY Subject keys - work.pty_rk                          *
	 *                  Incident keys - work.ras_rk                             *
	 * Target Table:    All Risk Assessment Incident to                         *
	 *                  Subject relationships -                                 * 
	 *                   work.all_ras_inc_x_pty                                 * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete all_ras_inc_x_pty;
	quit;

	proc sql;
	   create view work.all_ras_inc_x_pty as
	   select
	      ras_rk.INCIDENT_RK,
	      pty_rk.PARTY_RK
	   from
	      work.ras, 
	      work.pty_rk, 
	      work.ras_rk
	   where
	      ras.party_number = pty_rk.PARTY_ID
	      and ras.risk_assessment_id = ras_rk.INCIDENT_ID
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_RISK_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Table Loader                                            *
	 * Transform:       Table Loader (Version 2.1)                              * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    All Risk Assessment Incident to                         *
	 *                  Subject relationships -                                 * 
	 *                   work.all_ras_inc_x_pty                                 * 
	 * Target Table:    INCIDENT_X_PARTY -                                      *
	 *                   ecm_db.INCIDENT_X_PARTY                                * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.all_ras_inc_x_pty); 

	data _null_;
    	call symput('current_dt',put(datetime(),datetime22.));
	run;

    proc datasets lib = iem nolist nowarn memtype = (data view);
        delete new_inc_x_pty;
    quit;

	data iem.new_inc_x_pty;
		if 0 then set ecm_db.incident_x_party;
		set &SOURCEDS. (keep= INCIDENT_RK PARTY_RK);
		INCIDENT_RK = INCIDENT_RK;
		PARTY_RK = PARTY_RK;
		RELATION_TYPE_CD = "RSK2PTY";
		CREATE_DTTM = "&current_dt"dt;
	run;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_RISK_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	%let SOURCEDS = iem.new_inc_x_pty;
	    
	proc append base = ecm_db.INCIDENT_X_PARTY 
		data = &SOURCEDS force; 
	run; 

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_RISK_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 
	   
	/*==========================================================================* 
	 * Step:            Table Loader                                            *
	 * Transform:       Table Loader (Version 2.1)                              * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    All Risk Assessment Incident to                         *
	 *                  Subject relationships -                                 * 
	 *                   work.all_ras_inc_x_pty                                 * 
	 * Target Table:    ECM_EVENT - ecm_db.ECM_EVENT                            *
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.all_ras_inc_x_pty); 

	data _null_;
    	call symput('current_dt',put(datetime(),datetime22.));
	run;

    proc datasets lib = iem nolist nowarn memtype = (data view);
        delete new_events;
    quit;

	data iem.new_events;
		if 0 then set ecm_db.ecm_event;
		set &SOURCEDS. (keep= INCIDENT_RK PARTY_RK);
		BUSINESS_OBJECT_NM = "INCIDENT";
		BUSINESS_OBJECT_RK = INCIDENT_RK;
		EVENT_TYPE_CD = "ADDPAR";
		EVENT_DESC = "event.etl.load.txt";
		CREATE_USER_ID = "&ecm_user";
		CREATE_DTTM = "&current_dt"dt;
		LINKED_OBJECT_NM = "PARTY";
		LINKED_OBJECT_RK = PARTY_RK;
		drop INCIDENT_RK PARTY_RK;
	run;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_RISK_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	%let SOURCEDS = iem.new_events;
   
	%fcf_get_ecm_sequence_ids(in=iem.new_events,out=iem.new_events,type=event,block=&iem_performance_load);

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_RISK_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	proc append base = ecm_db.ECM_EVENT
		data = &SOURCEDS force; 
	run; 

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_RISK_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 
 
%MACRO_END:

%if &trans_rc eq 3 or &trans_rc ge 5 %then
    %fcf_execute_macro(fcf_error);
%else
    %do;
        %fcf_execute_macro(fcf_success);
    %end;


%mend;
