/******************************************************************************/
/*    NAME: cdd_sm_deploy_rules.sas                                                */
/* PRODUCT: SAS Customer Due Diligence                                        */
/*  SYSTEM: UNIX/WINDOWS                                                      */
/*    DESC: reads the package and updates the ECM database                    */
/* -------------------------------------------------------------------------- */
/* PARAMETERS:
  cdd_sm_rule_path - path where to download rule SAS code
  cdd_sm_rule_group_path  - path where to download rule group SAS code
  cdd_sm_codegen_path - path where to put generated SAS code
  cdd_deployment_id - unique suffix for the generated SAS CRR process code,
                      typically current timestamp
    rule_keys_tab_name - dataset name that contains party_rk for rules to deploy
    rule_group_keys_tab_name - dataset name that contains party_rk for rule groups to deploy
    deploy_type - KC deploy to KC
            T - test mode
*/
/******************************************************************************/
/*-------------------------------------------------------------------
 * Copyright (c) 2013 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro cdd_sm_deploy_rules(cdd_sm_rule_path=, cdd_sm_rule_group_path=, cdd_sm_codegen_path=, outlib=seg_kc,
                           cdd_deployment_id=,  rule_keys_tab_name=, rule_group_keys_tab_name=,
                           deploy_type=
);

  /* reset error codes to zero */
  %let trans_rc = 0;
  %let sqlrc = 0;
  %let sqlxrc = 0;
  %let sysrc = 0;
  %global current_dt;

  %macro cdd_sm_abort(msg=, p1=, p2=, p3=, p4=);
       %let parm_list=;
       %if "&p1" ne "" %then %do;  %let parm_list =, &p1;             %end;
       %if "&p2" ne "" %then %do;  %let parm_list =&parm_list , &p2;  %end;
       %if "&p3" ne "" %then %do;  %let parm_list =&parm_list , &p3;  %end;
       %if "&p4" ne "" %then %do;  %let parm_list =&parm_list , &p4;  %end;

       %let _fcfmsg_=ERROR: %sysfunc(sasmsg(sashelp.cddmsg, &msg, N &parm_list));
       %let trans_rc = 99;
       %fcf_put_error
       %cdd_clear_sync_lock()
       %ABORT CANCEL &trans_rc;
  %mend cdd_sm_abort;

  %macro cdd_sm_abort_on_error(msg=, p1=, p2=, p3=, p4=);
   %fcf_chkrc
   %if &bat_abort=Y %then %do;
       %let sys_err_msg = &bat_errmsg;
       %let parm_list=;
       %if "&p1" ne "" %then %do;  %let parm_list =, &p1;             %end;
       %if "&p2" ne "" %then %do;  %let parm_list =&parm_list , &p2;  %end;
       %if "&p3" ne "" %then %do;  %let parm_list =&parm_list , &p3;  %end;
       %if "&p4" ne "" %then %do;  %let parm_list =&parm_list , &p4;  %end;

       %let _fcfmsg_=ERROR: Update failed: %sysfunc(sasmsg(sashelp.cddmsg, &msg, N &parm_list));
       %let trans_rc = 99;
       %fcf_put_error
       %cdd_clear_sync_lock();
       %ABORT CANCEL &trans_rc;
   %end;
  %mend cdd_sm_abort_on_error;

  %macro cdd_sm_get_dav_url();
    /* assuming that DAVService URL has form:
       http://server99.foo.com:8080/SASWIPSoapServices/services/DAVService
       or
       https://server99.foo.com:8080/SASWIPSoapServices/services/DAVService
     */
      %svcreg(type=SOAP, name=DAVService);
      %let w1=%qkscan(&service_endpoint, 1, /);
      %if %upcase(&w1) eq HTTP: or %upcase(&w1) eq HTTPS: %then %do;
          %let w2=%qkscan(&service_endpoint, 2, /);
          %let fcf_dav_root=&w1//&w2/SASContentServer/repository/default/sasdav/;
          %put &fcf_dav_root;
      %end;
      %else %do;
          %cdd_sm_abort(msg=cdd_sm_dav_root_not_found, p1=&cdd_sync_job_ds)
      %end;
  %mend cdd_sm_get_dav_url;


  %macro cdd_sm_create_fsk_backups;
      libname cdd_back  "&cdd_sm_backup_folder";
      data cdd_back.fsk_cdd_rule (GENMAX=&cdd_sm_genmax);
           set seg_kc.fsk_cdd_rule;
      run;
      %cdd_sm_abort_on_error(msg=cdd_sm_cannot_backup_ds, p1=fsk_cdd_rule, p2=&cdd_sm_backup_folder)
      data cdd_back.fsk_cdd_rule_group (GENMAX=&cdd_sm_genmax);
           set seg_kc.fsk_cdd_rule_group;
      run;
      %cdd_sm_abort_on_error(msg=cdd_sm_cannot_backup_ds, p1=fsk_cdd_rule_group, p2=&cdd_sm_backup_folder)
      data cdd_back.fsk_cdd_rule_parameter (GENMAX=&cdd_sm_genmax);
           set seg_kc.fsk_cdd_rule_parameter;
      run;
      %cdd_sm_abort_on_error(msg=cdd_sm_cannot_backup_ds, p1=fsk_cdd_rule_parameter, p2=&cdd_sm_backup_folder)
  %mend cdd_sm_create_fsk_backups;

  %macro cdd_sm_setup();
    %cdd_sync_lock()
    %if &cdd_sync_lock_failed eq Y %then %do;
         %cdd_sm_abort(msg=cdd_critical_update_in_process, p1=&cdd_sync_job_ds)
    %end;
  %mend cdd_sm_setup;

  %macro cdd_sm_get_table_entities(tabName=, partyCat=, tabPrefix=, udfTabName=, charData=, numData=, dateData=, party_keys_table=, filter_deleted=);
      %global &tabPrefix._charRec &tabPrefix._numRec;
      %let &tabPrefix._charRec = 0;
      %let &tabPrefix._numRec = 0;

      %if &charData eq 1 %then %do;
         /* select all CHAR UDF columns for all columns */
          proc sql;
              create table &tabPrefix._CC1 as
              select  p.party_rk, c.udf_nm, c.udf_value length=&cdd_max_udf_len as UDF_VALUE, c.row_no
                      from ecm_db.party_live p
                      inner join ecm_db.party_udf_char_value c
                      on c.udf_table_nm = "&udfTabName"
                         and c.valid_from_dttm = p.valid_from_dttm and p.party_rk = c.party_rk
                      where p.party_category_cd = "&partyCat"
                      order by party_rk, row_no
              ;
          quit;
          proc sql;
                create table &tabPrefix._CC as
                select  * from &tabPrefix._CC1
                where party_rk in (select party_rk from &party_keys_table)
                ;
          quit;
          %let &tabPrefix._charRec = &SQLOBS;
          %cdd_sm_abort_on_error(msg=sa_processing_error, p1=&tabPrefix._CC)

          proc transpose data=&tabPrefix._CC out=&tabPrefix._CT;
               id UDF_NM;
               var UDF_VALUE;
               by PARTY_RK ROW_NO;
          run;
          data &tabPrefix._C;
              set &tabPrefix._CT;
              drop _LABEL_  _NAME_;
          run;
      %end;

      %if &numData eq 1 %then %do;
          /* select all NUMERIC UDF columns for all columns */
          proc sql;
              create table &tabPrefix._NC1 as
              select  p.party_rk, n.udf_nm, n.udf_value, n.row_no
                      from ecm_db.party_live p
                      inner join ecm_db.party_udf_num_value n
                      on n.udf_table_nm = "&udfTabName"
                         and n.valid_from_dttm = p.valid_from_dttm and p.party_rk = n.party_rk
                      where p.party_category_cd = "&partyCat"
                      order by party_rk, row_no
              ;
          quit;
          proc sql;
                create table &tabPrefix._NC as
                select  * from &tabPrefix._NC1
                where party_rk in (select party_rk from &party_keys_table)
                ;
          quit;
          %let &tabPrefix._numRec = &SQLOBS;
          %cdd_sm_abort_on_error(msg=sa_processing_error, p1=&tabPrefix._NC)

          /* pivot the table */
          proc transpose data=&tabPrefix._NC out=&tabPrefix._NT;
               id UDF_NM;
               var UDF_VALUE;
               by PARTY_RK ROW_NO;
          run;

          data &tabPrefix._N;
              set &tabPrefix._NT;
              drop _LABEL_  _NAME_;
          run;
          %if &charData eq 1 %then %do;
              /* join all UDF columns in one table */
              proc sql;
                  create table &tabPrefix._ALL_CN
                      as select c.*, n.*
                      from &tabPrefix._C c
                      left join &tabPrefix._N n
                          on c.party_rk = n.party_rk
                             AND c.row_no = n.row_no
                  ;
              quit;
          %end;
          %else %do;
              proc sql;
                  create table &tabPrefix._ALL_CN
                      as select n.*
                      from &tabPrefix._N n
                  ;
              quit;
          %end;
      %end;
      %else %do;
              proc sql;
                  create table &tabPrefix._ALL_CN
                      as select c.*
                      from &tabPrefix._C c
                  ;
              quit;
      %end;

      %if &dateData eq 1 %then %do;
          /* select all DATE UDF columns for all columns */
          proc sql;
              create table &tabPrefix._DC1 as
              select  p.party_rk, n.udf_nm, n.udf_value, n.row_no
                      from ecm_db.party_live p
                      inner join ecm_db.party_udf_date_value n
                      on n.udf_table_nm = "&udfTabName"
                         and n.valid_from_dttm = p.valid_from_dttm and p.party_rk = n.party_rk
                      where p.party_category_cd = "&partyCat"
                      order by party_rk, row_no
              ;
          quit;
          proc sql;
                create table &tabPrefix._DC as
                select  * from &tabPrefix._DC1
                where party_rk in (select party_rk from &party_keys_table)
                ;
          quit;
          %let &tabPrefix._dateRec = &SQLOBS;
          %cdd_sm_abort_on_error(msg=sa_processing_error, p1=&tabPrefix._DC)

          /* pivot the table */
          proc transpose data=&tabPrefix._DC out=&tabPrefix._DT;
               id UDF_NM;
               var UDF_VALUE;
               by PARTY_RK ROW_NO;
          run;

          data &tabPrefix._D;
              set &tabPrefix._DT;
              drop _LABEL_  _NAME_;
          run;
          %if &charData eq 1 or &numData eq 1 %then %do;
              /* join all UDF columns in one table */
              proc sql;
                  create table &tabPrefix._ALL
                      as select  n.*, d.*
                      from &tabPrefix._ALL_CN n
                      left join &tabPrefix._D d
                          on d.party_rk = n.party_rk
                             AND d.row_no = n.row_no
                  ;
              quit;
          %end;
          %else %do;
              proc sql;
                  create table &tabPrefix._ALL
                      as select d.*
                      from &tabPrefix._D d
                  ;
              quit;
          %end;
      %end;
      %else %do;
              proc sql;
                  create table &tabPrefix._ALL
                      as select d.*
                      from &tabPrefix._ALL_CN d
                  ;
              quit;
      %end;

    %if &filter_deleted = Y %then %do;
        proc sql;
          create table &tabName as
            select  a.*, p.PARTY_ID, p.VERSION_NO, p.valid_from_dttm
            from &tabPrefix._ALL a
            left join ecm_db.party_live p
            on a.party_rk = p.party_rk
            where CDD_LOGICAL_DELETE_FLG ne "1"
          ;
        quit;
    %end;
    %else %do;
        proc sql;
          create table &tabName as
            select  a.*, p.PARTY_ID, p.VERSION_NO, p.VALID_FROM_DTTM
            from &tabPrefix._ALL a
            left join ecm_db.party_live p
            on a.party_rk = p.party_rk
          ;
        quit;
      %end;
  %mend cdd_sm_get_table_entities;

  /****************************************************************************
   ****************************************************************************
   **         Setup the environment
   ****************************************************************************
   ****************************************************************************/

  %cdd_sm_get_dav_url;
  %cdd_sm_setup;

  /* create data set to store errors */
  data errors;
      length msg $400;
      stop;
  run;
  data _null_;
     call symput('current_dt',put(datetime(),datetime22.));
  run;
  /* some of the code above may set SYSRC to a negative value, so reset */
  %let sysrc = 0;

  /****************************************************************************
   ****************************************************************************
   **         Exctract data from ecm
   ****************************************************************************
   ****************************************************************************/
  /* -------------- get rules --------------- */
  %cdd_sm_get_table_entities (tabName=CDD_RULE_PART, partyCat=&cdd_sm_rule_cat, tabPrefix=rule, udfTabName=PARTY,   charData=1, numData=1, dateData=1, party_keys_table=&rule_keys_tab_name, filter_deleted=Y)
  /* -------------- get rule groups --------------- */
  %cdd_sm_get_table_entities (tabName=FSK_CDD_RULE_GROUP_ALL, partyCat=&cdd_sm_rule_group_cat, tabPrefix=rg, udfTabName=PARTY,   charData=1, numData=1, dateData=1, party_keys_table=&rule_group_keys_tab_name, filter_deleted=Y)

  %macro cdd_sm_convert_to_kc();

    /* Filter only active, non-deleted rules and add reference to a group */
    proc sql;
        create table FSK_CDD_RULE
            as select r.*, x.PARTY_RK as CDDRG_RK, g.PARTY_ID as CDD_RULE_GROUP_ID, g.VERSION_NO as CDD_RULE_GROUP_VERSION_NO, g.CDD_RULE_GROUP_NAME
              from CDD_RULE_PART r
              left join ecm_db.party_x_party x
              on x.member_party_rk = r.party_rk
              inner join FSK_CDD_RULE_GROUP_ALL g
              on x.party_rk = g.party_rk
            where CDD_RULE_STATUS_CODE = "ACT"
            order by PARTY_ID;
        ;
    quit;

    data FSK_CDD_RULE;
      set FSK_CDD_RULE (rename=(PARTY_ID=CDD_RULE_ID VALID_FROM_DTTM=CDD_UPDATE_DATE));
      length CURRENT_IND $1
             CDD_DEPLOY_DATE 8
      ;
      CURRENT_IND = 'Y';
      CDD_DEPLOY_DATE = "&current_dt"dt;
    run;

    /* validate that each rule is inlcuded only in one group */
    proc sql;
        create table rule_err as
      select r.* from FSK_CDD_RULE r
      where (select count(*) from FSK_CDD_RULE rr where r.CDD_RULE_ID = rr.CDD_RULE_ID) > 1
      order by CDD_RULE_ID;
        quit;

        %if &SQLOBS > 0 %then %do;
      %let cdd_deploy_validation_err_cnt = &SQLOBS;
          data rule_errors;
            set rule_err;
            by CDD_RULE_ID;
          length msg group_list $400;
            retain group_list;
            keep msg;
            if first.CDD_RULE_ID then group_list = kstrip(CDD_RULE_GROUP_NAME);
                       else group_list = cat(kstrip(group_list), ', ', kstrip(CDD_RULE_GROUP_NAME));
            if last.CDD_RULE_ID then do;
              msg = sasmsg("SASHELP.CDDMSG", "CDD_SM_RULE_INCLUDED_IN_TWO_GROUPS", "noquote", kstrip(CDD_RULE_NAME), kstrip(CDD_RULE_ID), kstrip(group_list));
              put msg;
              put CDD_RULE_NAME;
              put group_list;
              output;
            end;
          run;
          proc append base=errors data=rule_errors FORCE;
          run;
           %goto exit;
        %end;

    /* -------------- get rule parameters --------------- */
    /* do not extract parameters for deleted, inactive rules */
    proc sql;
        create table rule_param_keys as
            select party_rk  from work.FSK_CDD_RULE
      ;
    quit;
    %cdd_sm_get_table_entities (tabName=FSK_CDD_RULE_PARAMETER, partyCat=&cdd_sm_rule_cat, tabPrefix=rp, udfTabName=X_CDD_RULE_PARAM,   charData=1, numData=1, dateData=0, party_keys_table=rule_param_keys, filter_deleted=N)

    data FSK_CDD_RULE_PARAMETER;
      set FSK_CDD_RULE_PARAMETER (rename=(PARTY_ID=CDD_RULE_ID VERSION_NO=CDD_RULE_VERSION_NO VALID_FROM_DTTM=CDD_UPDATE_DATE));
    run;

    /* delete empty groups - they are not used by CRR process
       */
    proc sql;
      create table work.FSK_CDD_RULE_GROUP as
      select * from work.FSK_CDD_RULE_GROUP_ALL g
      where (select count(*) from fsk_cdd_rule r where r.CDDRG_RK = g.party_rk) > 0;
      quit;

    data FSK_CDD_RULE_GROUP;
      set FSK_CDD_RULE_GROUP (rename=(PARTY_ID=CDD_RULE_GROUP_ID VALID_FROM_DTTM=CDD_UPDATE_DATE));
      length CDD_RULE_GROUP_FILE_NAME $128;
      CDD_RULE_GROUP_FILE_NAME = trim(left(CDD_RULE_GROUP_ID)) || '_v_' || trim(left(VERSION_NO)) || '.sas';
      length CURRENT_IND $1;
      CURRENT_IND = 'Y';
      length CDD_DEPLOY_DATE 8;
      CDD_DEPLOY_DATE = "&current_dt"dt;
    run;

    %exit:
  %mend cdd_sm_convert_to_kc;

  %cdd_sm_convert_to_kc();

  %if &cdd_deploy_validation_err_cnt ne 0 %then %do;
    put ERROR: validation errors;
         %goto exit;
  %end;

  /****************************************************************************
   ****************************************************************************
   **          Download rule code
   ****************************************************************************
   ****************************************************************************/
  %macro cdd_sm_download_rules(dav_server_root=, dav_server_path=, rule_ds=);
      data _null_;
        set &rule_ds;
            locFileName = "&cdd_sm_rule_path" || trim(left(CDD_RULE_ID)) || '_v_' || trim(left(VERSION_NO)) || '.sas';
            davFileName = "&dav_server_root&dav_server_path" || 'r' || trim(left(PARTY_RK)) || '.sas';
            call execute ('%nrstr(%fcf_sa_get_file_from_dav_server(davFileName=' || davFileName ||', localFileName=' || locFileName || '))');
      run;
  %mend cdd_sm_download_rules;

  %cdd_sm_download_rules(dav_server_root=&fcf_dav_root, dav_server_path=&cdd_rules_dav_path, rule_ds=FSK_CDD_RULE)

  /****************************************************************************
   ****************************************************************************
   **          Download rule groups code
   ****************************************************************************
   ****************************************************************************/
  %macro cdd_sm_download_rule_groups(dav_server_root=, dav_server_path=, rule_group_ds=);
      data _null_;
        set &rule_group_ds;
              locFileName = "&cdd_sm_rule_group_path" || 'h' || trim(left(PARTY_RK)) || '_v_' || trim(left(VERSION_NO))|| '.sas';
              davFileName = "&dav_server_root&dav_server_path" || 'h' || trim(left(PARTY_RK)) || '.sas';
              call execute ('%nrstr(%fcf_sa_get_file_from_dav_server(davFileName=' || davFileName ||', localFileName=' || locFileName || '))');
  /* TODO: uncomment when ECM supports more than one FileEditor on a page
              locFileName = "&cdd_sm_rule_group_path" || 'f' || trim(left(PARTY_RK)) || '_v_' || trim(left(VERSION_NO)) || '.sas';
              davFileName = "&dav_server_root&dav_server_path" || 'f' || trim(left(PARTY_RK)) || '.sas';
              call execute ('%nrstr(%fcf_sa_get_file_from_dav_server(davFileName=' || davFileName ||', localFileName=' || locFileName || '))');
  */
      run;
  %mend cdd_sm_download_rule_groups;
  %cdd_sm_download_rule_groups(dav_server_root=&fcf_dav_root, dav_server_path=&cdd_rule_groups_dav_path, rule_group_ds=FSK_CDD_RULE_GROUP)

  /****************************************************************************
   ****************************************************************************
   **          Generate code
   ****************************************************************************
   ****************************************************************************/
  %macro cdd_cg_gen_rule_macro(inlib=, outlib=);
      proc sort data=&inlib..FSK_CDD_RULE_PARAMETER out=&outlib..cdd_rule_with_parm;
          by PARTY_RK ROW_NO;
      run;
      /* create parameter list and factual parameters */
      data  rule_param_list;
          set &outlib..cdd_rule_with_parm;
          by party_rk;
          length param_dcl $ 1024;
          length param_val $ 1024;
          keep party_rk param_dcl param_val;
          retain param_dcl param_val;
          if first.party_rk then do;
              param_dcl= trim(left(CDD_RULE_PARM_NAME)) || "=";
              if    CDD_RULE_PARM_TYPE_CODE = 'CHARL' or CDD_RULE_PARM_TYPE_CODE = 'CHARLIST'
                 or CDD_RULE_PARM_TYPE_CODE = 'NUML' or CDD_RULE_PARM_TYPE_CODE = 'EXPR' then
                param_val= trim(left(CDD_RULE_PARM_NAME)) || '=%str(' || trim(left(CDD_RULE_PARM_VAL)) || ")";
            else
                param_val= trim(left(CDD_RULE_PARM_NAME)) || "=" || trim(left(CDD_RULE_PARM_VAL));
          end;
          else do;
              param_dcl = trim(left(param_dcl)) || ", " || trim(left(CDD_RULE_PARM_NAME)) || "=";
              if   CDD_RULE_PARM_TYPE_CODE = 'CHARL' or CDD_RULE_PARM_TYPE_CODE = 'CHARLIST'
                or CDD_RULE_PARM_TYPE_CODE = 'NUML' or CDD_RULE_PARM_TYPE_CODE = 'EXPR' then
                  param_val = trim(left(param_val)) || ", " || trim(left(CDD_RULE_PARM_NAME)) || '=%str(' || trim(left(CDD_RULE_PARM_VAL)) || ")";
              else
                  param_val = trim(left(param_val)) || ", " || trim(left(CDD_RULE_PARM_NAME)) || "=" || trim(left(CDD_RULE_PARM_VAL)) ;

              put param_dcl;
              put param_val;
          end;
          if last.party_rk then do;
             output;
             param_dcl = '';
             param_val = '';
          end;
      run;

    /* add parameters for codegen, exclude rules that are not included in a group */
      proc sql;
          create table &outlib..cdd_rule_with_parm
              as select r.*, p.param_dcl, p.param_val, g.CDD_RULE_GROUP_INPUT_DS_NAME, g.CDD_RULE_GROUP_CATEGORY_CODE, g.CDD_RULE_GROUP_FILE_NAME
              from &inlib..FSK_CDD_RULE r
              left join rule_param_list p
              on r.party_rk = p.party_rk
              inner join FSK_CDD_RULE_GROUP g
              on r.CDDRG_RK = g.party_rk
          ;
      quit;

      proc sort data=&outlib..cdd_rule_with_parm;
          by CDDRG_RK CDD_RULE_ORDER_IN_GROUP;
      quit;

      data _null_;
          set &outlib..cdd_rule_with_parm;
          by CDDRG_RK;
        length rulesFilename $200;
        length locFileName $200;
        rulesFilename = "&cdd_sm_codegen_path.cdd_rules_" || trim(left("&cdd_deploy_date..sas"));
        file rg1 filevar=rulesFilename;
        locFileName = "&cdd_sm_rule_path" || trim(left(CDD_RULE_ID)) || '_v_' || trim(left(VERSION_NO)) || '.sas';

        put   '/* Rule: ' CDD_RULE_NAME  '       Last updated: ' CDD_UPDATE_DATE '*/';
        put   '%macro rule_' CDD_RULE_ID +(-1) '( ' PARAM_DCL ');';
        put   '    %cdd_rule_definition(cdd_rule_id=' CDD_RULE_ID +(-1) ', version_no=' VERSION_NO +(-1) ', cdd_rule_type=' CDD_RULE_TYPE_CODE +(-1) ',';
        put   '                         file_name=' locFileName +(-1) ', cdd_rule_save_id_flg=' CDD_RULE_SAVE_ID_FLG  +(-1) ',';
        put   '                         category_code=' CDD_RULE_GROUP_CATEGORY_CODE +(-1) ', input_ds_name=' CDD_RULE_GROUP_INPUT_DS_NAME  +(-1);
        put   '                        );';
        put   '%mend rule_' CDD_RULE_ID ';';
        put   ' ';
      run;
  %mend cdd_cg_gen_rule_macro;

  %macro cdd_cg_gen_rule_group(inlib=, outlib=);
      /* generate all other rules in the group */
      data _null_;
          set &outlib..cdd_rule_with_parm;
          by CDDRG_RK;
        length groupFilename $255;
        length locFileName $255;
        if first.CDDRG_RK then do;
          groupFilename = "&cdd_sm_codegen_path" || trim(left(CDD_RULE_GROUP_FILE_NAME));
        end;
        file rg1 filevar=groupFilename;

        if first.CDDRG_RK then do;
           locFileName = "&cdd_sm_rule_group_path" || 'h' || trim(left(CDDRG_RK)) || '_v_' || trim(left(CDD_RULE_GROUP_VERSION_NO)) || '.sas';
           put   '/* Group: ' CDD_RULE_GROUP_NAME '         Last updated: ' CDD_UPDATE_DATE '*/';
           put   ' ';
           put   '/* group header */';
           put   '%inc "' locFileName +(-1) '";';
        end;

        put   ' ';
        put   '/* Rule: ' CDD_RULE_NAME '*/';
        /* rules operating on non-merged prep data sets also ouput cotegory code */
        put   '%rule_' CDD_RULE_ID  '( ' param_val ');';
        if last.CDDRG_RK then do;
           locFileName = "&cdd_sm_rule_group_path" || 'f' || trim(left(CDDRG_RK)) || '_v_' || trim(left(CDD_RULE_GROUP_VERSION_NO)) || '.sas';;
           put   ' ';
           put   '/*  group footer */';
           /* FIXME: workaround */
           put   '/* %inc "' locFileName +(-1) '"; */';
           /* FIXME: workaround */
           put   'run;';
        end;
      run;
  %mend cdd_cg_gen_rule_group;

  %cdd_cg_gen_rule_macro(inlib=work, outlib=work);
  %cdd_cg_gen_rule_group(inlib=work, outlib=work);

  /****************************************************************************
   ****************************************************************************
   **         Backup existing tables before updating them
   ****************************************************************************
   ****************************************************************************/
  %if &deploy_type = KC %then %do;
    %cdd_sm_create_fsk_backups
    %end;

  /****************************************************************************
   ****************************************************************************
   **         Update KC table
   ****************************************************************************
   ****************************************************************************/
  %let worklib=work;
  %macro cdd_sm_update_kc_tables();
    /*
    *	FSK_CDD_RULE update logic:
    *		mark all KC records that are not in ECM as CURRENT_IND = N (deleted rules, rules that are not in any group, inactive rules)
    *		mark all KC record that have different VERSION_NO than rules in ECM as CURRENT_IND = N (old rule version)
    *		insert ECM records that have different VERSION_NO than KC or have CDD_RULE_ID  that is not in KC as CURRENT_IND = Y  this are updated and new rules
    */

      proc sql;
        update seg_kc.fsk_cdd_rule r
           set CURRENT_IND = 'N'
        where  (select count(*) from &worklib..fsk_cdd_rule e where r.cdd_rule_id = e.cdd_rule_id and r.CURRENT_IND ="Y") = 0
               or
           (select count(*) from &worklib..fsk_cdd_rule e where r.cdd_rule_id = e.cdd_rule_id and r.version_no ne e.version_no and r.CURRENT_IND ="Y") > 0
        ;
      quit;

      /* set the current_ind back to Y for the rules that have the same version - this may happen if the group was deleted and the rule
       * was marked as non-current, then group was restored and rule should be current again but version in ECM has not changed, so no new
       * records will be inserted in fsk_cdd_rule, old record should become current again
       */
      proc sql;
        update seg_kc.fsk_cdd_rule r
           set CURRENT_IND = 'Y'
        where (select count(*) from &worklib..fsk_cdd_rule e where r.cdd_rule_id = e.cdd_rule_id and r.version_no = e.version_no and r.CURRENT_IND ="N") > 0
        ;
      quit;

      /* now insert new rule records, either new rules or rules with new version  */
    proc sql;
       create table &worklib..new_rule_records as
           select * from &worklib..fsk_cdd_rule r
           where (select count(*) from seg_kc.fsk_cdd_rule s where r.cdd_rule_id = s.cdd_rule_id and r.version_no ne s.version_no and s.CURRENT_IND ="Y") > 0
                 or
                 (select count(*) from seg_kc.fsk_cdd_rule s where r.cdd_rule_id = s.cdd_rule_id and s.CURRENT_IND ="Y") = 0
       ;
    quit;

    proc append base=seg_kc.fsk_cdd_rule data=&worklib..new_rule_records FORCE;
    run;
      %cdd_sm_abort_on_error(msg=sa_processing_error, p1=seg_kc.fsk_cdd_rule)

    /*
    * FSK_CDD_RULE_GROUP update logic:
    *   mark all KC records that are not in ECM as CURRENT_IND = N (deleted rule groups or empty rule groups)
    *	mark all KC record that have different VERSION_NO than rule groups in ECM as CURRENT_IND = N (old rule group version)
    *	insert ECM records that have different VERSION_NO than KC or have CDD_RULE_GROUP_ID  that is not in KC as CURRENT_IND = Y  this are updated and new rule groups
    */

      proc sql;
        update seg_kc.fsk_cdd_rule_group r
           set CURRENT_IND = 'N'
        where  (select count(*) from &worklib..fsk_cdd_rule_group e where r.cdd_rule_group_id = e.cdd_rule_group_id and r.CURRENT_IND ="Y") = 0
               or
           (select count(*) from &worklib..fsk_cdd_rule_group e where r.cdd_rule_group_id = e.cdd_rule_group_id and r.version_no ne e.version_no and r.CURRENT_IND ="Y") > 0
        ;
      quit;

      /* set the current_ind back to Y for the rule groups that have the same version - this may happen if the group became empty in ECM, so it
       * was marked as non-current, then a rule was added to the group and rule group should be current again but version in ECM has not changed, so no new
       * records will be inserted in fsk_cdd_rule_group, old record should become current again
       */
      proc sql;
        update seg_kc.fsk_cdd_rule_group r
           set CURRENT_IND = 'Y'
        where (select count(*) from &worklib..fsk_cdd_rule_group e where r.cdd_rule_group_id = e.cdd_rule_group_id and r.version_no = e.version_no and r.CURRENT_IND ="N") > 0
        ;
      quit;

    proc sql;
       create table &worklib..new_rule_group_records as
           select * from &worklib..fsk_cdd_rule_group r
           where (select count(*) from seg_kc.fsk_cdd_rule_group s where r.cdd_rule_group_id = s.cdd_rule_group_id and r.version_no ne s.version_no and s.CURRENT_IND ="Y") > 0
                 or
                 (select count(*) from seg_kc.fsk_cdd_rule_group s where r.cdd_rule_group_id = s.cdd_rule_group_id and s.CURRENT_IND ="Y") = 0
       ;
    quit;

    proc append base=seg_kc.fsk_cdd_rule_group data=&worklib..new_rule_group_records FORCE;
    run;
      %cdd_sm_abort_on_error(msg=sa_processing_error, p1=seg_kc.fsk_cdd_rule_group)

    /*
    * FSK_CDD_RULE_PARAMETER  update logic
    *	insert ECM records that for parameters for new rules
    */

    proc sql;
       create table &worklib..new_params as
           select * from &worklib..fsk_cdd_rule_parameter r
           where ( select count(*) from &worklib..new_rule_records n where n.cdd_rule_id = r.cdd_rule_id ) > 0
       ;
    quit;


    proc append base=seg_kc.fsk_cdd_rule_parameter data=&worklib..new_params FORCE;
    run;
    %cdd_sm_abort_on_error(msg=sa_processing_error, p1=seg_kc.fsk_cdd_rule_parameter)
  %mend cdd_sm_update_kc_tables;

  %if &deploy_type = KC %then %do;
    %cdd_sm_update_kc_tables
  %end;

  %if &deploy_type = T %then %do;
    %if %kupcase(&outlib) ne WORK %then %do;
      data &outlib..FSK_CDD_RULE;
        set work.FSK_CDD_RULE;
      run;
      data &outlib..FSK_CDD_RULE_GROUP;
        set work.FSK_CDD_RULE_GROUP;
      run;
      data &outlib..FSK_CDD_RULE_PARAMETER;
        set work.FSK_CDD_RULE_PARAMETER;
      run;
    %end;
  %end;

  %macro cdd_run_crr_codegen ();
    %let cdd_codegen_success_flg=0;
    %cdd_crr_codegen(  cdd_codegen_path=&cdd_sm_codegen_path
                     , cdd_rule_libname=&outlib
                     , cdd_deployment_id=&cdd_deployment_id
             , cdd_codegen_success_flg_mvar=cdd_codegen_success_flg
                    );
    %if &cdd_codegen_success_flg ne 1 %then %do;
          %cdd_sm_abort(msg=crr_codegen_err)
    %end;
  %mend cdd_run_crr_codegen;

  %cdd_run_crr_codegen;

%exit:
%mend cdd_sm_deploy_rules;